SystemOrganization addCategory: #Parsec!
SystemOrganization addCategory: 'Parsec-Lexing'!
SystemOrganization addCategory: 'Parsec-Visual'!

!Object methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
printRecursivelyAsSelfEvaluatingFormOn: aStream

	^ self printOn: aStream! !

Object subclass: #ParsecAlgo
	instanceVariableNames: 'readingPosition dummyRule grammar input charts logging string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecAlgo commentStamp: 'NickPapoulias 11/30/2016 14:48' prior: 0!
Early algo parses as EBNF!

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
alternativesFor: aState
		
	^ aState rightHandSideAsNewStatesOn: aState to! !

!ParsecAlgo methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
atChartAndStateEnd

	"return both indicators"

	^ charts atChartAndStateEnd! !

!ParsecAlgo methodsFor: 'changing suspended state' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
complete: aState

	aState rule name == #gamma ifTrue: [ 
		charts successfullTrees add: aState ].
	
	charts copyStatesWaitingFor: aState 
			from: aState from"previousReadingPosition"
			to: aState to.
			
	"aState alternativeSelf do: [ :alternative | self complete: alternative ]"
! !

!ParsecAlgo methodsFor: 'initialization' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
initialize

	super initialize.

	readingPosition := 0.
	dummyRule := (#gamma --> #start , #Eof) asGammaStart.
	charts := ParsecCharts new.
	logging := false.
	
! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
log

	logging := true.
	charts log.
! !

!ParsecAlgo methodsFor: 'parsing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
parse: aString using: aGrammar 

	string := aString. 
	input := aString readStream.
	grammar := aGrammar.
	dummyRule grammar: grammar.
	charts currentAdd: 
		(ParsecState on: dummyRule alternative: 1 from: 0 to: 0 dot: 0).
	(grammar at: #ignore) regExpr matchesStreamPrefix: input. "always at the start of next token"
	
	readingPosition := input position.
	
	"proceed returns true while we have more charts to loop through"
	[charts currentDo: 
		[:state | self totAlgoOn: state]; proceed] whileTrue.

	^ charts
	 ! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
predict: aState

	"(aState expectingAnd or: [ aState expectingNot ]) 
		ifTrue: [ ^ self forkToHandle: aState ]."
	

	charts currentAddAll: (self alternativesFor: aState)
		! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
scan: aState

	|match |
	
	aState expectingEmpty ifTrue: [ ^ self scanEmpty: aState ].
	aState expectingEof ifTrue: [ ^ self scanEof: aState ].

	match := aState ruleAtDot 
		terminalMatchForInput: input
		atPosition: aState to.
			
	(match > aState to)
		ifTrue: [ 
			charts nextAdd: 
				(aState asScannerStateFrom: readingPosition to: match withIndex: 1). 
				"always 1 for terminals"
			readingPosition := match.
			^ true
		].
	
	^ false! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
scanEmpty: aState

	charts nextEmptyAdd: 
		(aState asScannerStateFrom: readingPosition to: readingPosition withIndex: 1).
		
	^ true
	
! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
scanEof: aState

	^ input size = aState to
		ifTrue: [ 
			charts nextAdd: 
				(aState asScannerStateFrom: readingPosition to: readingPosition withIndex: 1). true ]
		ifFalse: [ false ]	
! !

!ParsecAlgo methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 15:12'!
totAlgoOn: state

	logging ifTrue: [ Transcript show: state; cr. ].

	state	isIncomplete
		ifTrue: [   
			state expectingTerminal
				ifTrue: [ self scan: state ]
				ifFalse: [ self predict: state ] 
			]
		ifFalse: [ self complete: state ].
! !

ParsecAlgo subclass: #ParsecOrderedAlgo
	instanceVariableNames: 'backTrackStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecOrderedAlgo commentStamp: 'NickPapoulias 11/30/2016 14:38' prior: 0!
EarleyPeg Algo parses as PEG (adds fork and backtrack for supporting \ & not) but every choice is ordered!

ParsecOrderedAlgo subclass: #ParsecMixedOrderAlgo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecMixedOrderAlgo commentStamp: 'NickPapoulias 11/30/2016 14:49' prior: 0!
MixedForm algo parses as a mix of EBNF and Peg (supporting in equal footing ordered and unordered choice, and, not etc) we can say that this is a form of EPEG (in analogy with EBNF)!

!ParsecMixedOrderAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
predict: aState

	aState ruleAtDot isOrdered
		ifTrue: [ super predict: aState ]
		ifFalse: [ super predictUnordered: aState ]! !

ParsecMixedOrderAlgo subclass: #ParsecMixedOrderFilteredAlgo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecMixedOrderFilteredAlgo methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 22:12'!
alternativesFor: aState
	
	"inlined super and filter alternatives"
	
	| newStates rule |
	
	newStates := ParsecCollection new.
	rule := aState ruleAtDot.
	
	1 to: rule rightHandSide size do: [ :i | "we can use either predict either first here"
		((rule directRefsAt: i) first isNot | (self check: (rule predict at: i) at: aState to))
				ifTrue: [  
					newStates
						add:
						((ParsecState
							on: aState ruleAtDot
							alternative: i
							from: aState to
							to: aState to
							dot: 0)
							fromTerminal: aState toTerminal;
							toTerminal: aState toTerminal) ] 
	].
				
	^ newStates
! !

!ParsecMixedOrderFilteredAlgo methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 15:21'!
check: predict at: position

	1 to: predict size do: [ :i | |pRule|
		pRule := predict at: i. 
		pRule isEmptyOrEof
			ifTrue: [ ^ true ]
			ifFalse: [ (self try: pRule at: position) ifTrue: [ ^ true ] ] ].
	^ false! !

!ParsecMixedOrderFilteredAlgo methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 15:22'!
try: pRule at: position

	^ (pRule
		terminalMatchForInput: input
		atPosition: readingPosition) > position! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
alternativesFor: aState
		
	^ aState rightHandSideAsNewStatesOn: aState to! !

!ParsecOrderedAlgo methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
backTrack
	| btState |
	
	(backTrackStack isNotEmpty and: [ self atChartAndStateEnd ])
		ifFalse: [ ^ self ].
	btState := backTrackStack pop.
	readingPosition := btState readingPosition.
	charts
		backTrackAt: btState chartIndex -> btState stateIndex
		with: btState state! !

!ParsecOrderedAlgo methodsFor: 'adding' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
backTrackAdd: alternative

				backTrackStack push: 
					(ParsecBackTrackState new 
						at: readingPosition indexedBy: charts chartIndex at: charts current size with: alternative)! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
complete: aState

	"aState is complete delete stack alternatives otherwise this is a full backtrack"
	self removeStackAlternativesOf: aState rule startingAt: aState from.
	super complete: aState.! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
forkParse: aStream at: position using: aGrammar

	input := aStream.
	grammar := aGrammar. "can be optimized"
	dummyRule grammar: grammar.
	charts currentAdd: 
		(ParsecState on: dummyRule alternative: 1 from: 0 to: 0 dot: 0).
		
	(grammar at: #ignore) regExpr matchesStreamPrefix: input. "always at the start of next token"
	
	readingPosition := position.
	
	charts current last start: readingPosition; end: readingPosition. "dummy rule starts at fork" 
	
	"proceed returns true while we have more charts to loop through"
	[charts currentDo: 
		[:state | self totAlgoOn: state]; proceed] whileTrue.

	^ (charts last detect: [ :state | state rule name = #gamma ]) to " on NotFound ? "
	 ! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 15:15'!
forkToHandle: aState

	|resultPosition forkedGrammar |
	
	"optimizaiton: retrieve memoized terminals"
	(forkedGrammar := grammar copy)
		at: aState ruleAtDot name put: aState ruleAtDot copy beVanilla;
		at: #start put: ((#start --> aState ruleAtDot name) asGammaStart firsts: aState ruleAtDot firsts).

	resultPosition := 
		(logging
			ifTrue: [ self class new log ]
			ifFalse: [ self class new ]) 
			forkParse: input 
			at: readingPosition 
			using: forkedGrammar.
	input position: readingPosition. "restore"
	charts printToLog.
	(resultPosition > readingPosition) "succeed"
			ifTrue: [ aState expectingAnd 
								ifTrue: [ aState advanceDot. self totAlgoOn: aState ]
								ifFalse: [ self backTrack "expected not but succeeded" ] ]
			ifFalse: [ aState expectingNot
								ifTrue: [ aState advanceDot. self totAlgoOn: aState ] "failed with not"
								ifFalse: [ self backTrack ] ] "failed with and"		
! !

!ParsecOrderedAlgo methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	super initialize.
	backTrackStack := ParsecStack new.! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
predict: aState

	|alter|
	
	(aState expectingAnd or: [ aState expectingNot ]) 
		ifTrue: [ ^ self forkToHandle: aState ].
	
	alter := self alternativesFor: aState.
	alter isEmpty ifTrue: [ ^ self ]. "this normally can only happen through TFOAlgo -- fix
	see with the non-peg example"
	(charts currentAdd: alter first)
		ifTrue: [ "could be duplicate"2 to: alter size  do: [:i| |a| a := (alter at: i). self backTrackAdd: a]
		] ifFalse: [  
			|added|
			2 to: alter size do: [ :i | |a|
				a := alter at: i. 
				added ifNotNil: [ self backTrackAdd: a ]
						ifNil: [ (charts currentAdd: a) 
										ifTrue: [ added := a ]  
										ifFalse: [  self backTrackAdd: a ]
								]
				]
		] "duplicate"
	! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
predictUnordered: aState

	(aState expectingAnd or: [ aState expectingNot ]) 
		ifTrue: [ ^ self forkToHandle: aState ].

	^ super predict: aState

! !

!ParsecOrderedAlgo methodsFor: 'removing' stamp: ' 1/8/2017 13:28:27'!
removeStackAlternativesOf: aRule startingAt: from
	
	"optimize -- normally this should remove one or more at the top -- or can be a dict of stacks"
	|btState|
	[
		backTrackStack isEmpty 
			ifTrue: [ false ]
			ifFalse: [  
				btState := backTrackStack top.
				btState state rule == aRule and: [ btState state from == from]
			]
	] whileTrue: [ backTrackStack pop ].
	"(backTrackStack 
		select: [:btState | btState state rule = aRule and: [ btState state from = from ]  ])
		do: [ :btState | backTrackStack remove: btState ifAbsent: [  ] ]"
! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
scan: aState

	(super scan: aState)
		ifFalse: [self backTrack]

	
! !

!ParsecOrderedAlgo methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
scanUnordered: aState

	super scan: aState
		
	
! !

Object subclass: #ParsecBackTrackState
	instanceVariableNames: 'chartIndex stateIndex readingPosition state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecBackTrackState commentStamp: 'NickPapoulias 11/1/2016 18:21' prior: 0!
Implement as a stack resume to the last after a failed scan!

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
at: inputPosition indexedBy: chart at: index with: aState

	state := aState.
	chartIndex := chart.
	stateIndex := index.
	readingPosition := inputPosition.
	! !

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
at: inputPosition indexedBy: indexPair with: aState

	state := aState.
	chartIndex := indexPair key.
	stateIndex := indexPair value.
	readingPosition := inputPosition.
	! !

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
chartIndex

	^ chartIndex

	! !

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
readingPosition

	^ readingPosition

	! !

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
state

	^ state

	! !

!ParsecBackTrackState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
stateIndex

	^ stateIndex

	! !

Object subclass: #ParsecBenchMarks
	instanceVariableNames: 'name example ppJsonParser ppJsonMemoParser ppcJsonParser ppcJsonMemoParser totJsonParser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecBenchMarks methodsFor: 'initialization' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
initialize

	super initialize.
	ppJsonParser := PPJsonGrammar new.
	ppJsonMemoParser := PPJsonGrammar new memoized transform: [ :each | each memoized ].
	ppcJsonParser := PP2JsonGrammar new.
	ppcJsonMemoParser := PP2JsonGrammar new memoize transform: [ :each | each memoize ].
	totJsonParser := ParsecJsonGrammar new.! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
jsonExample

	^ self perform: (example ifNil: [ example := #jsonExampleBig ])! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
jsonExampleBig

	"http://beta.json-generator.com/NyJDGFM-f"

	^ '[
  {
    "_id": "5829933f45655c137f72afb4",
    "index": 0,
    "guid": "1de3c7fa-9f5f-4ce2-bd00-1492485feea5",
    "isActive": true,
    "balance": "$3,545.59",
    "picture": "http://placehold.it/32x32",
    "age": 26,
    "eyeColor": "green",
    "name": {
      "first": "Reeves",
      "last": "Stanton"
    },
    "company": "CHILLIUM",
    "email": "reeves.stanton@chillium.io",
    "phone": "+1 (993) 405-2896",
    "address": "114 Lombardy Street, Tivoli, New York, 5382",
    "about": "Nulla eiusmod sunt cupidatat officia laborum exercitation duis aliquip Lorem laboris nostrud veniam. Sunt in magna consequat reprehenderit exercitation veniam ut consequat exercitation. Occaecat nisi aliquip ad exercitation dolore. Id laboris pariatur aliquip irure ut duis et sunt dolore dolore. Exercitation non aliqua excepteur sint magna laborum. Tempor velit quis ad in Lorem Lorem amet mollit voluptate duis reprehenderit sunt dolor. Dolor qui enim qui mollit labore sunt fugiat.",
    "registered": "Saturday, November 28, 2015 10:53 PM",
    "latitude": "-73.020167",
    "longitude": "134.786528",
    "tags": [
      "adipisicing",
      "irure",
      "velit",
      "in",
      "pariatur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Mccarthy Mcguire"
      },
      {
        "id": 1,
        "name": "Sadie Swanson"
      },
      {
        "id": 2,
        "name": "Hodge Moody"
      }
    ],
    "greeting": "Hello, Reeves!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f561d35c3308fbcd8",
    "index": 1,
    "guid": "f2601a6c-4999-44fb-aeb2-af7849905ca4",
    "isActive": false,
    "balance": "$2,760.73",
    "picture": "http://placehold.it/32x32",
    "age": 30,
    "eyeColor": "brown",
    "name": {
      "first": "Patti",
      "last": "Aguirre"
    },
    "company": "HYDROCOM",
    "email": "patti.aguirre@hydrocom.ca",
    "phone": "+1 (999) 589-2632",
    "address": "837 Adler Place, Brazos, New Mexico, 6246",
    "about": "Magna dolore in nulla esse sint occaecat excepteur deserunt culpa excepteur. Anim quis minim fugiat fugiat anim laborum. Pariatur culpa non id aliqua eu anim sint ad adipisicing tempor non adipisicing laboris. Dolore adipisicing pariatur sit occaecat ut. Aliqua dolor minim excepteur laborum cupidatat nisi est cupidatat ut exercitation. Incididunt est commodo esse ea sint reprehenderit est labore laboris ex minim proident Lorem do.",
    "registered": "Wednesday, October 19, 2016 5:22 PM",
    "latitude": "38.995324",
    "longitude": "-95.68088",
    "tags": [
      "sint",
      "amet",
      "sunt",
      "consectetur",
      "Lorem"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Briggs Chapman"
      },
      {
        "id": 1,
        "name": "Kelley Neal"
      },
      {
        "id": 2,
        "name": "Chandra William"
      }
    ],
    "greeting": "Hello, Patti!! You have 6 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fc55451f6efea29e2",
    "index": 2,
    "guid": "9eed2015-63f5-469b-bc4d-3f625bb742d0",
    "isActive": false,
    "balance": "$2,077.11",
    "picture": "http://placehold.it/32x32",
    "age": 31,
    "eyeColor": "blue",
    "name": {
      "first": "Clemons",
      "last": "Jarvis"
    },
    "company": "COMTRAK",
    "email": "clemons.jarvis@comtrak.org",
    "phone": "+1 (910) 404-2741",
    "address": "258 Dakota Place, Summerset, South Carolina, 2662",
    "about": "Cillum magna tempor voluptate consectetur ullamco enim mollit ullamco aliquip incididunt nisi cupidatat. Nostrud aute est nisi incididunt cupidatat ut dolor ad non nostrud. Dolore labore fugiat consequat consectetur incididunt cupidatat minim laboris. Ullamco est est incididunt nulla nulla minim enim occaecat fugiat ut tempor. Deserunt cupidatat nisi adipisicing est velit id amet qui. Commodo aliqua tempor occaecat ea consectetur consectetur.",
    "registered": "Monday, December 1, 2014 7:42 AM",
    "latitude": "22.819718",
    "longitude": "-159.31187",
    "tags": [
      "deserunt",
      "incididunt",
      "ipsum",
      "enim",
      "velit"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Little Rodgers"
      },
      {
        "id": 1,
        "name": "Sofia Cooley"
      },
      {
        "id": 2,
        "name": "Monroe Jennings"
      }
    ],
    "greeting": "Hello, Clemons!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f376548679c1f7fe3",
    "index": 3,
    "guid": "6e22aeee-d6ee-433f-977c-7bd1acd10273",
    "isActive": false,
    "balance": "$1,811.95",
    "picture": "http://placehold.it/32x32",
    "age": 32,
    "eyeColor": "green",
    "name": {
      "first": "Mathis",
      "last": "Leach"
    },
    "company": "COMTREK",
    "email": "mathis.leach@comtrek.me",
    "phone": "+1 (941) 469-2252",
    "address": "885 Pooles Lane, Morriston, Indiana, 1144",
    "about": "Anim voluptate id nulla officia irure. Anim sunt qui elit ipsum consequat nisi. Minim laborum eiusmod officia nostrud. Duis irure laborum sit deserunt nostrud occaecat pariatur ut consectetur incididunt quis tempor consequat aliquip. Eu excepteur qui fugiat voluptate sint pariatur aliqua commodo ea fugiat aliquip ipsum nulla. Aliquip enim anim commodo pariatur dolor eiusmod fugiat culpa aliquip labore esse officia labore.",
    "registered": "Tuesday, July 15, 2014 12:27 AM",
    "latitude": "-86.086873",
    "longitude": "-145.063901",
    "tags": [
      "aliqua",
      "nisi",
      "cupidatat",
      "voluptate",
      "enim"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Nixon Underwood"
      },
      {
        "id": 1,
        "name": "Sandoval Oneil"
      },
      {
        "id": 2,
        "name": "Jodie Bass"
      }
    ],
    "greeting": "Hello, Mathis!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f91080e43f2815a8f",
    "index": 4,
    "guid": "a110cbcd-da9c-44c5-9740-12795daa38a1",
    "isActive": true,
    "balance": "$3,948.74",
    "picture": "http://placehold.it/32x32",
    "age": 38,
    "eyeColor": "brown",
    "name": {
      "first": "Hardy",
      "last": "Pace"
    },
    "company": "EMOLTRA",
    "email": "hardy.pace@emoltra.biz",
    "phone": "+1 (823) 523-3553",
    "address": "150 Varick Street, Crawfordsville, Rhode Island, 1521",
    "about": "Amet sunt occaecat officia laboris exercitation. Id in amet nostrud eiusmod esse elit. Ullamco laborum occaecat ullamco ipsum qui dolore est pariatur mollit magna nostrud dolore cillum. Excepteur dolore ad consectetur magna pariatur eiusmod. Veniam exercitation proident deserunt magna. Ad consequat deserunt reprehenderit commodo nulla adipisicing id pariatur consequat exercitation do ullamco proident dolore.",
    "registered": "Saturday, February 20, 2016 3:42 AM",
    "latitude": "75.28317",
    "longitude": "176.558437",
    "tags": [
      "ex",
      "quis",
      "ex",
      "qui",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Macdonald Cleveland"
      },
      {
        "id": 1,
        "name": "Caldwell Brewer"
      },
      {
        "id": 2,
        "name": "Justine Beasley"
      }
    ],
    "greeting": "Hello, Hardy!! You have 9 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fe9ef1f88060d2e73",
    "index": 5,
    "guid": "5e034438-ae3a-4981-bc30-c07ccb4ba2c2",
    "isActive": false,
    "balance": "$3,161.10",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "blue",
    "name": {
      "first": "Martha",
      "last": "Spence"
    },
    "company": "COLLAIRE",
    "email": "martha.spence@collaire.net",
    "phone": "+1 (894) 513-2129",
    "address": "788 Crooke Avenue, Gardiner, Connecticut, 4632",
    "about": "Cupidatat fugiat id sint deserunt in et et aliquip qui velit do reprehenderit occaecat et. Nulla nostrud enim id anim irure elit enim fugiat nostrud voluptate est consequat. Voluptate laboris labore occaecat aliquip dolor. Voluptate ad fugiat laboris quis aute deserunt in aliqua.",
    "registered": "Wednesday, August 6, 2014 11:53 AM",
    "latitude": "21.811418",
    "longitude": "71.300384",
    "tags": [
      "incididunt",
      "pariatur",
      "cupidatat",
      "ea",
      "veniam"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Brown Williams"
      },
      {
        "id": 1,
        "name": "Deanna Ferguson"
      },
      {
        "id": 2,
        "name": "Violet Shaw"
      }
    ],
    "greeting": "Hello, Martha!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fc77a2bae9dc5145a",
    "index": 6,
    "guid": "e33f729a-d690-49ba-bc89-ca56f934e0ea",
    "isActive": false,
    "balance": "$3,451.86",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "brown",
    "name": {
      "first": "Stacy",
      "last": "Houston"
    },
    "company": "INSURON",
    "email": "stacy.houston@insuron.com",
    "phone": "+1 (854) 435-3498",
    "address": "959 Concord Street, Aurora, New Hampshire, 7772",
    "about": "Velit exercitation excepteur eiusmod exercitation aliqua duis cupidatat eiusmod aliquip pariatur laborum amet. Laborum adipisicing officia culpa enim pariatur fugiat id officia magna. Sit occaecat Lorem aliquip sint id laborum magna. Consectetur labore laborum reprehenderit magna nostrud adipisicing. Lorem deserunt Lorem sunt dolor. Dolore amet ut commodo nulla Lorem incididunt Lorem sunt labore laborum nisi consectetur.",
    "registered": "Tuesday, November 24, 2015 11:38 AM",
    "latitude": "77.866637",
    "longitude": "-57.468626",
    "tags": [
      "non",
      "esse",
      "velit",
      "deserunt",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Kimberley Mayer"
      },
      {
        "id": 1,
        "name": "Hampton Joyner"
      },
      {
        "id": 2,
        "name": "Bowen Freeman"
      }
    ],
    "greeting": "Hello, Stacy!! You have 9 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fee2454e42a23756c",
    "index": 7,
    "guid": "6d5eeb9c-1c9b-4c94-9ade-7b85927e6d89",
    "isActive": true,
    "balance": "$3,923.75",
    "picture": "http://placehold.it/32x32",
    "age": 33,
    "eyeColor": "blue",
    "name": {
      "first": "Constance",
      "last": "Travis"
    },
    "company": "ZIDOX",
    "email": "constance.travis@zidox.biz",
    "phone": "+1 (918) 499-2733",
    "address": "243 Malbone Street, Wheaton, Iowa, 8798",
    "about": "Aliquip proident est et sunt magna. Duis ipsum velit laboris reprehenderit excepteur veniam. Sunt occaecat excepteur excepteur labore. Culpa sit aute ut id laborum commodo sint do qui enim magna ex anim. Voluptate ullamco veniam minim voluptate aute duis elit sunt voluptate in nisi dolore sit. Elit labore laboris amet incididunt in cupidatat non et est qui consectetur.",
    "registered": "Wednesday, January 20, 2016 9:41 PM",
    "latitude": "73.648824",
    "longitude": "-150.40766",
    "tags": [
      "duis",
      "est",
      "labore",
      "sint",
      "aute"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Terri Mccarthy"
      },
      {
        "id": 1,
        "name": "Amalia Blankenship"
      },
      {
        "id": 2,
        "name": "Salas Oneill"
      }
    ],
    "greeting": "Hello, Constance!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fbdb1a902f9ed2736",
    "index": 8,
    "guid": "fa4ab5e7-b357-4da7-9a1e-33103721f4ad",
    "isActive": true,
    "balance": "$3,439.45",
    "picture": "http://placehold.it/32x32",
    "age": 34,
    "eyeColor": "blue",
    "name": {
      "first": "Thelma",
      "last": "Woods"
    },
    "company": "ACCUSAGE",
    "email": "thelma.woods@accusage.co.uk",
    "phone": "+1 (983) 512-2649",
    "address": "116 Gunnison Court, Idledale, West Virginia, 1593",
    "about": "Adipisicing quis qui Lorem est dolore elit qui Lorem sit aliqua. Sint duis veniam ipsum ex pariatur culpa nisi ut proident deserunt Lorem. Nisi enim non velit excepteur commodo non consectetur voluptate dolor quis mollit.",
    "registered": "Wednesday, July 23, 2014 7:59 AM",
    "latitude": "46.749703",
    "longitude": "-30.646582",
    "tags": [
      "sit",
      "veniam",
      "ipsum",
      "do",
      "consectetur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Lynnette Tanner"
      },
      {
        "id": 1,
        "name": "Lynn Thompson"
      },
      {
        "id": 2,
        "name": "Kristen Manning"
      }
    ],
    "greeting": "Hello, Thelma!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  }
]'! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
jsonExampleSmall


	^ '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
ppJson
	
	^ [  1 to: self times do: [ :i |	ppJsonParser parse: self jsonExample]] timeToRun! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
ppJsonMemo
		
		^ [  1 to: self times do: [ :i |	ppJsonParser parse: self jsonExample]] timeToRun! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
ppcJson
		 
		^ [  1 to: self times do: [ :i |	ppcJsonParser parse: self jsonExample]] timeToRun! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
ppcJsonMemo

		^ [  1 to: self times do: [ :i |	ppcJsonParser parse: self jsonExample]] timeToRun! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
run

	^ (self totJson / self ppJson) asFloat! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
runFilteredWithPPC
		
	self use: ParsecMixedOrderFilteredAlgo.

	^ (self totJson / self ppcJson) asFloat! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
runWithMemo

	^ (self totJson / self ppJsonMemo) asFloat! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
runWithPPC

	^ (self totJson / self ppcJson) asFloat! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
runWithPPCMemo

	^ (self totJson / self ppcJsonMemo) asFloat! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
times

	^ 1000! !

!ParsecBenchMarks methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
totJson

		^ [  1 to: self times do: [ :i |	totJsonParser parse: self jsonExample]] timeToRun! !

!ParsecBenchMarks methodsFor: 'initialization' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
use: anAlgoClass

	totJsonParser with: anAlgoClass! !

!ParsecBenchMarks methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: aSelector

	example := aSelector! !

Object subclass: #ParsecCollection
	instanceVariableNames: 'array currentSize totalSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

ParsecCollection subclass: #ParsecCharts
	instanceVariableNames: 'chartIndex stateIndex logging current successfullTrees'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecCharts class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
new

	^ super new initialize! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
atChartAndStateEnd

	^ self size == chartIndex and: [ self current size = stateIndex ]! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
atEnd

	^ self size == chartIndex! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
augment

	self add: ParsecCollection new.
! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
backTrackAt: indexPair with: aNewState

	"should update currentDo: also"
	
	logging ifTrue: [
		Transcript
			show: 'BT: [Chart' , chartIndex asString , '(' , stateIndex asString , ')' ,  ']';  
			show: '======================'; cr.
	].
	chartIndex := indexPair key. current := self at: chartIndex.
	stateIndex := indexPair value - 1. "replay"
	chartIndex + 1 to: self size do: [:i | self removeIndex: i  ]. "remove"
	stateIndex + 2 to: self current size do: [ :i | self current removeIndex: i ].
	(self at: chartIndex) at: stateIndex + 1 put: aNewState.! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
chartIndex 

	^ chartIndex! !

!ParsecCharts methodsFor: 'copying' stamp: 'NickPapoulias 4/12/2017 13:57'!
copyStatesWaitingFor: aState from: start to: end

	| name chart element newState |

	name := aState rule name.

	
	chart := (self at: aState fromTerminal + 1).

    1 to: chart size do: [ :index |
		element := chart at: index.
		(element expecting: name at: start)
			ifTrue: [  |childrenDict|
				newState := element copy.
				"newState previousStates add: aState."
				newState fromTerminal: element fromTerminal; toTerminal: chartIndex - 1.
				newState advanceDot end: end.
				childrenDict := (self currentAddWithState: newState) childrenStates. "will return either the newState or the identical pre-existing"
				childrenDict ~= newState childrenStates "already exists transfer children"
					ifTrue: [ newState childrenStates keysAndValuesDo: [:k :values |  
							values do: [ :v |
								|col| col := (childrenDict at: k ifAbsent: [ childrenDict at: k put: OrderedCollection new ]).
								col detect: [ :p | p equalTo: v ] ifNone: [ col add: v].
							] 			
					]].
				"aState from ~= newState from ifTrue: [  
					self assert: (childrenDict values flatten select: [ :v | v to = aState from ]) isEmpty not.
				]."
				(childrenDict at: aState from ifAbsent: [ childrenDict at: aState from put: OrderedCollection new ])
					add: aState. "children from multiple trees"  
				
			]
	].! !

!ParsecCharts methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
current

	^ current! !

!ParsecCharts methodsFor: 'adding' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
currentAdd: aState

	"optimized version without anySatisfy.. where are my macros"
	
	|prev|
	
	1 to: current size do: [ :i | ((current at: i) equalTo: aState) ifTrue: [ ^ false ]  ].
	prev := self previous.
	1 to: prev size do: [ :i | ((prev at: i) equalTo: aState) ifTrue: [ ^ false ]  ].
	current add: aState. ^ true
! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
currentAddAll: states

	1 to: states size do: [:i | self currentAdd: (states at: i)]! !

!ParsecCharts methodsFor: 'adding' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
currentAddWithState: aState

	"optimized version without anySatisfy.. where are my macros"
	
	|prev|
	
	1 to: current size do: [ :i | ((current at: i) equalTo: aState) ifTrue: [ ^ current at: i ]  ].
	prev := self previous.
	1 to: prev size do: [ :i | ((prev at: i) equalTo: aState) ifTrue: [ ^ prev at: i ]  ].
	^ current add: aState.
! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
currentDo: aBlock

	self printToLog.

	stateIndex := 0. "chart may change while looping so using indexing"
	[ stateIndex < self current size ] whileTrue: [
			stateIndex := stateIndex + 1. 
			aBlock value: (self current at: stateIndex).
	].

! !

!ParsecCharts methodsFor: 'initialization' stamp: 'NickPapoulias 1/19/2017 14:56'!
initialize

	super initialize.
	chartIndex := 1.
	logging := false.
	successfullTrees := ParsecCollection new.
	self augment.
	current := self at:1.! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
log

	logging := true! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
next

	chartIndex + 1 > totalSize ifTrue: [ ^ {  } ].
	
	^ (self at: chartIndex + 1) "no nil collection" 
		ifNil: [ {  } ]! !

!ParsecCharts methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
nextAdd: aState
	"optimized version without anySatisfy.. where are my macros"
	
	|current next|
	
	next := self next.
	1 to: next size do: [ :i | ((next at: i) equalTo: aState) ifTrue: [ ^ self ]  ].
	current := self current.
	1 to: current size do: [ :i | ((current at: i) equalTo: aState) ifTrue: [ ^ self ]  ].
	self atEnd ifTrue: [self augment]. self next add: aState. ! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
nextEmptyAdd: aState
	self atEnd
		ifFalse: [ ^ self ].
	self augment.
	self next add: aState! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
previous

	chartIndex == 1 ifTrue: [ ^ {  } ].
	chartIndex - 1 > totalSize ifTrue: [ ^ {  } ].
	
	^ (self at: chartIndex - 1) "no nil collection" 
		ifNil: [ {  } ]

! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
printToLog
	logging
		ifTrue: [ Transcript
				show: '[Chart' , chartIndex asString;
				show: ']';
				show: '======================';
				cr ]! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/16/2017 23:17:18'!
proceed

	^ (chartIndex := chartIndex + 1) <= self size
			ifTrue: [ current := self at: chartIndex. true ]
			ifFalse: [ false ]! !

!ParsecCharts methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
stateIndex 

	^ stateIndex! !

!ParsecCharts methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
succeeded

	^ self last anySatisfy: [ :state | state isFinal ]! !

!ParsecCharts methodsFor: 'initialization' stamp: 'NickPapoulias 1/19/2017 14:56'!
successfullTrees

	^ successfullTrees! !

!ParsecCollection methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
add: anItem

	currentSize := currentSize + 1.
	currentSize > totalSize
		ifTrue: [ self grow ].
	^ array at: currentSize put: anItem! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
addAll: aCollection 

	aCollection do: [:each | self add: each].
	^ aCollection! !

!ParsecCollection methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
addLast: anItem

	^ self add: anItem! !

!ParsecCollection methodsFor: 'enumerating' stamp: 'NickPapoulias 1/16/2017 13:15'!
anySatisfy: aBlock

	1 to: currentSize do: [:i | (aBlock value: (self at: i)) ifTrue: [^ true]].
	^ false! !

!ParsecCollection methodsFor: 'adding' stamp: 'NickPapoulias 1/17/2017 14:57'!
array

	^ array! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
array: aCopy

	array := aCopy! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
at: anIndex

	^ array at: anIndex "we can go with no check, should use ifAbsent for nil values"
	! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
at: anIndex ifAbsent: aBlock
	
	"self halt." "some erratic behaviour on profiling concerning the block value, inlined"
	
	anIndex == 0 ifTrue: [ ^ aBlock value ].
	anIndex > totalSize ifTrue: [ ^ aBlock value ].
	
	^ (self at: anIndex) "no nil collection" 
		ifNil: [ aBlock value ]! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
at: anIndex put: anItem
	
	^ array at: anIndex put: anItem

	! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/17/2017 20:17'!
atLast: index

	^ array at: currentSize - index! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
collect: aBlock

	|newCollection|
	newCollection := self class new.
	self do: [ :item | newCollection add: (aBlock value: item) ].
	^ newCollection! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
collectWithIndex: elementAndIndexBlock
		"Just like with:collect: except that the iteration index supplies the second argument to the block."
	| result |
	result := self class new.
	1 to: self size do:
		[:index | result at: index put:
		(elementAndIndexBlock
			value: (self at: index)
			value: index)].
	^ result! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
copy

	^ super copy array: array copy! !

!ParsecCollection methodsFor: 'copying' stamp: ' 1/8/2017 13:28:27'!
copyEmpty

	^ self class new
	! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
deepFlatten

	^ Array streamContents: [ :stream |
			self do: [ :each | each deepFlattenInto: stream ]]! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
deepFlattenInto: stream

	self do: [ :each | each deepFlattenInto: stream ]! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
detect: aBlock

	1 to: currentSize do: [:i | (aBlock value: (self at: i)) ifTrue: [ ^ (self at: i) ] ].
	NotFound signalFor: aBlock! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
do: aBlock

	1 to: currentSize do: [:i | (aBlock value: (self at: i)) ]! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 15:50'!
do: elementBlock separatedBy: separatorBlock
	"Evaluate the elementBlock for all elements in the receiver,
	and evaluate the separatorBlock between."

	| beforeFirst | 
	beforeFirst := true.
	self do:
		[:each |
		beforeFirst
			ifTrue: [beforeFirst := false]
			ifFalse: [separatorBlock value].
		elementBlock value: each]! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
doWithIndex: aBlock 
		
	1 to: self size do:
		[:index |
		aBlock
			value: (self at: index)
			value: index]! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
fifth

	^ self at: 5! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
first

	^ self at: 1! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
fourth

	^ self at: 4! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
grow

	|newArray newSize|
	newSize := totalSize * 2.
	newArray := Array new: newSize.
	newArray replaceFrom: 1 to: totalSize with: array startingAt: 1.
	totalSize := newSize.
	array := newArray.! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 16:11'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: 0>  
		
	^ (composite list)
		title: 'Items';
		display: [ :aCollection | array ];
		beMultiple;
		withSmalltalkSearch;		
		format: [ :each | GTObjectPrinter new asTruncatedTextFrom: each ];
		send: [ :result | result isNil 
			ifTrue:[nil]
			ifFalse:[ (result size = 1) 
				ifTrue: [result anyOne] 
				ifFalse: [self species withAll: result]]];
		showOnly: 50;
		helpMessage: 'Quick selection field. Given your INPUT, it executes: self select: [:each | INPUT ]'.! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/17/2017 13:19'!
ifEmpty: aBlock

	^ self isEmpty 
		ifTrue: [ aBlock value ]
		ifFalse: [ self ]! !

!ParsecCollection methodsFor: 'adding' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
includes: anObject 

	^ self anySatisfy: [:each | each = anObject]! !

!ParsecCollection methodsFor: 'initialization' stamp: 'NickPapoulias 1/16/2017 22:42'!
initialize

	currentSize := 0. 
	totalSize := 5. "can grow"
	array := Array new: totalSize.
	! !

!ParsecCollection methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
isEmpty

	^ currentSize == 0! !

!ParsecCollection methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
isNotEmpty

	^ (currentSize == 0) not! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
last

	^ array at: currentSize! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
nineth

	^ self at: 9! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 15:52'!
printElementsOn: aStream
	"The original code used #skip:, but some streams do not support that,
	 and we don't really need it."

	aStream nextPut: $(.
	self do: [:element | aStream print: element] separatedBy: [aStream space].
	aStream nextPut: $)! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 16:12'!
printNameOn: aStream
	
	aStream 
		nextPutAll: 'a Collection [';
		nextPutAll: currentSize asString;
		nextPutAll: ' items';
		nextPutAll: '] ' ! !

!ParsecCollection methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 15:55'!
printOn: aStream 
	"Append a sequence of characters that identify the receiver to aStream."

	self printNameOn: aStream.
	self printElementsOn: aStream! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
removeIndex: anIndex
	"too costly ?"
	array 
		replaceFrom: anIndex 
		to: currentSize - 1 
		with: array 
		startingAt: anIndex+1.
	array at: currentSize put: nil.	
	currentSize := currentSize - 1.
! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
second

	^ self at: 2! !

!ParsecCollection methodsFor: 'adding' stamp: 'NickPapoulias 1/16/2017 12:12'!
select: aBlock 

	| newCollection |
	newCollection := self class new.
	self do: [ :each | 
		(aBlock value: each) 
			ifTrue: [ newCollection add: each ]].
	^newCollection! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
seventh

	^ self at: 7! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
sixth

	^ self at: 6! !

!ParsecCollection methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
size

	^ currentSize! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
tenth

	^ self at: 10! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
third

	^ self at: 3! !

!ParsecCollection methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
withIndexCollect: elementAndIndexBlock 
	"Just like with:collect: except that the iteration index supplies the second argument to the block."
	| result |
	result := self class new.
	1 to: self size do:
		[:index | result at: index put:
		(elementAndIndexBlock
			value: (self at: index)
			value: index)].
	^ result! !

ParsecCollection subclass: #ParsecStack
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecStack methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
pop

	| ret |
	ret := self at: currentSize.
	self at: currentSize put: nil.
	currentSize := currentSize - 1.
	^ ret! !

!ParsecStack methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
push: anItem

	self add: anItem! !

!ParsecStack methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
top

	^ self at: currentSize! !

Object subclass: #ParsecDFAArray
	instanceVariableNames: 'array regExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecDFAArray methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
asGraph

	|nodesTo edges triples|
	triples := OrderedCollection new.
	array doWithIndex: [ :row :index | 
		
		nodesTo := (row asSet select: [ :item | (#(nil true false) includes: item) not ]) asOrderedCollection.
		nodesTo do: [:n | 
			edges := OrderedCollection new.
			row doWithIndex: [ :column :i | column = n ifTrue: [edges add: (Character value: i-1)]].
			triples add: ( ('n' , index asString) -> (edges size = 1 ifTrue: [ edges first ] ifFalse: [  edges asArray]) -> ('n' , n asString))  ]
		].
		 
	^ triples

	! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
asString

	^ regExpr! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
match: aString

	^ self matchesStreamPrefix: aString readStream
	! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: 'Nick Papoylias 1/16/2017 16:33:48'!
matchesStreamPrefix: s

	|index initPosition endPos c row|
	index := 1.
	initPosition := s position.
	row := array at: index. 
	[ 
		(row at: 257) ifTrue: [ endPos := s position ]. "next may proceed without succeeding"
		(c := s next) 
			ifNil: [ false ] 
			ifNotNil: [(index := row at: c asciiValue + 1) isNotNil]  
	] whileTrue: [ row := array at: index].
	
	^ endPos 
		ifNil: [ s directSetPos: initPosition. false] "maybe in a end state"
		ifNotNil: [ s directSetPos: endPos. true ]

	! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: ' 1/10/2017 20:14:21'!
regExpr

	^ regExpr! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: ' 1/10/2017 20:14:21'!
regExpr: aString

	regExpr := aString! !

!ParsecDFAArray methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: anArray

	array := anArray! !

Object subclass: #ParsecGraph
	instanceVariableNames: 'gEdges gElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Visual'!

!ParsecGraph class methodsFor: 'example' stamp: ' 1/8/2017 13:28:27'!
example

	^ ParsecGraph new 
	with: { 
		'a' -> '1' -> 'b' .  
		'b' -> '2' -> 'c' .
		'c' -> '3' -> 'a' .
		'b' -> '4' -> 'd' .
		'd' -> '5' -> 'c'
	};
	view! !

!ParsecGraph class methodsFor: 'example' stamp: ' 1/8/2017 13:28:27'!
example2

	|result|
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	^ (ParsecGraph new with: result asGraph) viewWithLayoutCoefficient: 2! !

!ParsecGraph class methodsFor: 'example' stamp: ' 1/8/2017 13:28:27'!
example3

	|result|
	result := (ParsecLexer new parse: '"(a|b)*"') asNFA.
	^ ((ParsecGraph new with: result asGraph) viewWithLayoutCoefficient: 3) view ! !

!ParsecGraph class methodsFor: 'example' stamp: ' 1/8/2017 13:28:27'!
example4

	|result|
	result := (ParsecLexer new parse: '"(a|b)*"') asDFA.
	^ ((ParsecGraph new with: result asGraph) viewWithLayoutCoefficient: 4) view ! !

!ParsecGraph class methodsFor: 'example' stamp: ' 1/8/2017 13:28:27'!
example5

	|result|
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"') asDFA.
	^ ((ParsecGraph new with: result asGraph) viewWithLayoutCoefficient: 4) view ! !

!ParsecGraph methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
addEdge: edge from: from to: to in: v updating: anchors
	
	| line e label  |
	line := RTDecoratedLine new 
			filledArrowHead;
			filledCircleTail;
			line: (RTBezier3Line new 
			width: [ :m | RTScale linear domain: #(60 100); range: #(2 6); scale: m fourth ]);
			attachPoint: (RTBorderAttachPoint new offset: 9);
			color: (Color colorFrom: edge third) "trans".
	e := RTEdge on: edge from: from to: to.
	e+line.
	v add: e.
	label := (RTLabel new elementOn: (edge fifth printStringLimitedTo: 20)) color: Color black; yourself.
				"v add: label."
	anchors add: (label -> (RTAnchorConstraint new
		element: label;
		edge: e;
		balance: 0.5;
		minDistance: 20))! !

!ParsecGraph methodsFor: 'instance creation' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
addElement: anElement childSelector: aChildSelector edgeSelector: anEdgeSelector

	(gElements includes: anElement) 
		ifFalse: [ 
			gElements add: anElement.
			gEdges addAll: (anElement perform: anEdgeSelector).
			(anElement perform: aChildSelector) do: [ :child | 
				self addElement: child childSelector: aChildSelector edgeSelector: anEdgeSelector  
			]
		]	
	

	! !

!ParsecGraph methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
colorFor: aString
	
	"good enough"
	^ Color fromRgbTriplet: ((1 to: 3) collect: [ :i | (aString hash digitAt: i) / 255.0]). ! !

!ParsecGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
edges

	^ gEdges! !

!ParsecGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
elements

	^ gElements ifNil: [ gElements := Set new.
		gEdges	do: [ :e | gElements add: e value; add: e key key ].
		gElements
	]
! !

!ParsecGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
elementsAsShapesOn: data

	^ (RTSelectiveShape new
		shape: #fifth;
		width: [ :m | RTScale linear domain: #(40 80); range: #(20 60); scale: m third.];
		text: #second;
		labelShapeDo: [ :shape | shape outlineWidth: 0.5; outlineColor: #fourth ];
		backgroundColor: #fourth;
		color: Color white) elementsOn: data.
! !

!ParsecGraph methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
propertiesForEdge: ed

	
		^ (Array with: ed key key with: ed value with: (self colorFor: ed key key) with: 90 with: ed key value)! !

!ParsecGraph methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
propertiesForElement: element

	^ (Array with: element with: element with: 75 with: (self colorFor: element) with: #ellipse)! !

!ParsecGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
view

	
	^ self viewWithLayoutCoefficient: 3
					! !

!ParsecGraph methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
viewWithLayoutCoefficient: forceCoefficient

	
	|anchors before edges elements v data  |

	anchors := OrderedCollection new.	
	v := RTView new.
	data := self elements 
		collect: [ :el | 
			self propertiesForElement: el"(Array with: e with: e with: 75 with: '#6FB1FC' with: #ellipse)" ]. 
	edges := self edges collect: [ :ed | self propertiesForEdge: ed ].
		"(Array with: ed key key with: ed value with: (self colorFor: ed key) with: 90 with: ed key value)"
	
	elements := self elementsAsShapesOn: data.
	v addAll: elements.
	elements @ RTDraggable.
	edges do: [ :edge| | from to line |
		from := elements detect: [ :el | el model first = edge first ] ifNone: [ nil ].
		to:= elements detect: [ :el | el model first = edge second ] ifNone: [ nil ].
		from = to ifTrue: [  |t id| "add an intermediate 0 sized node and connect through it"
			id := '.' , from model first asString.
			t := (self elementsAsShapesOn:  
					{(Array with: id with: '' with: 0 with: (self colorFor: '') with: #ellipse)}) first.
			t @ RTDraggable.
			v add: t.
			self addEdge: edge from: from to: t in: v updating: anchors.
			self addEdge: edge from: t to: to in: v updating: anchors.
		]
			ifFalse: [  
				self addEdge: edge from: from to: to in: v updating: anchors.
			]
	].

	v edges do: #pushBack.


	"elements first translateTo: 300@260.
	elements second translateTo: 170@150.
	elements third translateTo: 160@260.
	elements fourth translateTo: 290@150."
	v @ RTDraggableView @ RTZoomableView.
	(RTLayoutBuilder new forceWithCharge: -1*(10 raisedTo: forceCoefficient)*(v elements size)) on: (before := v elements).
	anchors do: [:a | v add: a key. a value build ].
	before do: [ :em | em translateBy: 0.001  ]. "hack to update edges"
	^ v.
					! !

!ParsecGraph methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: listOfTriples

	gEdges := listOfTriples! !

!ParsecGraph methodsFor: 'instance creation' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
withRootElement: anElement childSelector: aChildSelector edgeSelector: anEdgeSelector

	gElements := Set new. gEdges := Set new.
	self 
		addElement: anElement 
		childSelector: aChildSelector 
		edgeSelector: anEdgeSelector.

	! !

ParsecGraph subclass: #ParsecTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Visual'!

!ParsecTree class methodsFor: 'example' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
example

	"consider adding selectors that describe the relationships ie
	give {nodes} with: #edgeSelector to: #childSelector; edgeLabel: #edgeLabelSelector; nodeLabel: #nodeLabelSelector
	and parametrize the lable as well"

	|anObject|

	anObject := Object new.
	
	^ (ParsecTree new 
	with: { 
		anObject -> '1' -> 'b' .  
		anObject -> '2' -> 'c' .
		'c' -> '3' -> 'd' .
		'c' -> '4' -> 'e' .
		'b' -> '5' -> 'f'
	};
	view) open! !

!ParsecTree class methodsFor: 'example' stamp: 'NickPapoulias 3/27/2017 14:40'!
example2

	"consider adding selectors that describe the relationships ie
	give {nodes} with: #edgeSelector to: #childSelector; edgeLabel: #edgeLabelSelector; nodeLabel: #nodeLabelSelector 
	and parametrize the lable as well"

	| result root |
	result := ParsecExprGrammar new 
		parse: '2*3+4/1' using: ParsecMixedOrderAlgo new log.
	root := result charts last detect: [ :state | state isFinal ].
	"children := root perform: #childrenStates.
	edges := root perform: #childrenPositions."

	
	^ (ParsecTree new 
		withRootElement: root 
		childSelector: #childrenStates 
		edgeSelector: #childrenWithPositions;
		view) open! !

!ParsecTree methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
view

	
	^ self viewWithTreeLayoutWithGaps: 45.0 -> 50.0! !

!ParsecTree methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
viewWithTreeLayoutWithGaps: gapAssociation

	
	|anchors before edges elements v data |

	anchors := OrderedCollection new.	
	v := RTView new.
	data := self elements 
		collect: [ :el | 
			self propertiesForElement: el"(Array with: e with: e with: 75 with: '#6FB1FC' with: #ellipse)" ]. 
	edges := self edges collect: [ :ed | self propertiesForEdge: ed ].
		"(Array with: ed key key with: ed value with: (self colorFor: ed key) with: 90 with: ed key value)"
	
	elements := self elementsAsShapesOn: data.
	v addAll: elements.
	elements @ RTDraggable.
	edges do: [ :edge| | from to line |
		from := elements detect: [ :el | el model first = edge first ] ifNone: [ nil ].
		to:= elements detect: [ :el | el model first = edge second ] ifNone: [ nil ].
		from = to ifTrue: [  |t id| "add an intermediate 0 sized node and connect through it"
			id := '.' , from model first asString.
			t := (self elementsAsShapesOn:  
					{(Array with: id with: '' with: 0 with: (self colorFor: '') with: #ellipse)}) first.
			t @ RTDraggable.
			v add: t.
			self addEdge: edge from: from to: t in: v updating: anchors.
			self addEdge: edge from: t to: to in: v updating: anchors.
		]
			ifFalse: [  
				self addEdge: edge from: from to: to in: v updating: anchors.
			]
	].

	v edges do: #pushBack.


	"elements first translateTo: 300@260.
	elements second translateTo: 170@150.
	elements third translateTo: 160@260.
	elements fourth translateTo: 290@150."
	v @ RTDraggableView @ RTZoomableView.
(	RTLayoutBuilder new tree horizontalGap: gapAssociation key; verticalGap: gapAssociation value; yourself) on: (before := v elements).
	anchors do: [:a | v add: a key. a value build ].
	before do: [ :em | em translateBy: 0.001  ]. "hack to update edges"
	^ v.
					! !

Object subclass: #ParsecLexer
	instanceVariableNames: 'stream'
	classVariableNames: 'EscapeChars'
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
escapeChars

	^ EscapeChars ifNil: [  
		EscapeChars := Dictionary new.
		EscapeChars
			at: $\ put: $\;
			at: $/ put: $/;
			at: $" put: $";
			at: $b put: Character backspace;
			at: $f put: Character newPage;
			at: $n put: Character lf;
			at: $r put: Character cr;
			at: $t put: Character tab;
			yourself
		]! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parse: aString
	
	stream := aString readStream.
	^ self parseRe.! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseAny
	
	stream next.
	^ ParsecLexerAny new! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseAtom

	^ $( == stream peek
		ifTrue: [ |ret| stream next. ret := self parseRe. 
			stream next = $) ifFalse: [ self error: ') expected' ].
			ret 
		]
		ifFalse: [  
			|atom peek|
			atom := '' writeStream.
			peek := stream peek.
			$[ == peek ifTrue: [ ^ self parseGroup ].
			$. == peek ifTrue: [ ^ self parseAny ].
			$\ == peek ifTrue: [ ^ self parseEscape ].
			[(#($( $) $+ $* $| $? $[ $. $\ ${ nil) includes: stream peek)] 
				whileFalse: [ atom nextPut: stream next ].
			ParsecLexerOperand new: atom contents.
		]  ! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseConcatOp

	|operand|
	operand := self parseAtom.
	('+*?{' includes: stream peek) 
		ifTrue: [ ^ self parseUnaryOp: operand ]
		ifFalse: [ "flatOp optimization"
			((operand isKindOf: ParsecLexerOperand) ifTrue: [ ^ operand ] ifFalse: [ ^ ParsecLexerOperand new: operand ]).  ]! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseCountedRepetition

	|from to next|

	from := String new writeStream.
	to := String new writeStream.
	[ (next := stream next) isDigit ] whileTrue: [ from nextPut: next ].
	from := from contents asNumber.
	
	stream peek == $.
		ifTrue: [  
			stream next.
			[ (next := stream next) isDigit ] whileTrue: [ to nextPut: next ].
			to := to asNumber.		
		] ifFalse: [  
			to := from.	
		].
	
	^ ParsecLexerRepetition new between: from->to! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseEscape

	|escape|
	stream next.
	escape := stream next.
	^ ParsecLexerEscape new with: 
		([self escapeChars at: escape] on: NotFound do: [escape])! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseGroup

	stream next.	
	^ stream peek == $^ "not"
		ifTrue: [ stream next. ParsecLexerNotGroup new with: self parseGroupExpression ] "should return a lexer option"
		ifFalse: [ ParsecLexerGroup new with: self parseGroupExpression ]! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseGroupExpression

	|optionNode|
	
	optionNode := ParsecLexerOption new.
	[(stream peek == $]) not] whileTrue: [  
		stream peek = $\
			ifTrue: [ optionNode add: self parseEscape ]
			ifFalse: [ |start| start := stream next. 
				stream peek = $- 
					ifTrue: [ |finish| 
						stream next. 
						finish := stream next. 
						optionNode 
							add: (ParsecLexerRange new with: start -> finish)  ]
					ifFalse: [ optionNode add: (ParsecLexerOperand new with: start) ]
			]].
	stream next. "eat ]"
	^ optionNode size = 1 ifTrue: [ optionNode first ] ifFalse: [ optionNode ] "flatten"! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseOptionOp

	|result|
	result := ParsecLexerConcat new.
	result add: self parseConcatOp.
	[(#($| $) nil) includes: stream peek)] 
		whileFalse: [ result add: self parseConcatOp. ]. 
	^ result size = 1 ifTrue: [ result first ] ifFalse: [ result ] "flatten"! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseRe
	
	|result|
	result := ParsecLexerOption new.
	result add: self parseOptionOp.
	[stream peek = $|] 
		whileTrue: [ stream next. result add: self parseOptionOp. ].
	^ result size = 1 ifTrue: [ result first ] ifFalse: [ result ] "flatten"! !

!ParsecLexer methodsFor: 'parsing' stamp: ' 1/8/2017 13:28:27'!
parseUnaryOp: operand

	|op flatOp|
	op := stream next.
	"flatOp := (operand isKindOf: TotLexerOperand) ifTrue: [ operand operand ] ifFalse: [ operand ]. "
	flatOp := operand. "review flattening, cause it can change not-group semantics"
	"optimization: either string or composite" 
	op = $+ ifTrue: [ ^ ParsecLexerPlus new: flatOp ].
	op = $* ifTrue: [ ^ ParsecLexerStar new: flatOp ].
	op = $? ifTrue: [ ^ ParsecLexerOpt new: flatOp ].	
	op = ${ ifTrue: [ ^ self parseCountedRepetition with: flatOp ].! !

Object subclass: #ParsecLexerNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

ParsecLexerNode subclass: #ParsecLexerConcat
	instanceVariableNames: 'sequence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerConcat methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
add: aLexerNode

	^ sequence add: aLexerNode! !

!ParsecLexerConcat methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ (self sequence  collect: [ :item | item asGraph add: self hash -> ',' -> item hash; yourself ]) deepFlatten asOrderedCollection! !

!ParsecLexerConcat methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name. (self sequence collect: [ :item | item asList  ]) asArray } ! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
first

	^ sequence first! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
fourth

	^ sequence fourth! !

!ParsecLexerConcat methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	sequence := OrderedCollection new! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	sequence do: [ :node | (node lexMatch: aStream) ifFalse: [ ^ false ] ].
	^ true! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
second

	^ sequence second! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
sequence 

	^ sequence! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
size

	^ sequence size! !

!ParsecLexerConcat methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
third

	^ sequence third! !

!ParsecLexerConcat methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| last subGraph ret|
	ret := ParsecNFAGraph new.
	last := (self sequence at: 1) toNFA.
	ret startNode: last startNode.
	2 to: self sequence size do: [:i |
		subGraph := (self sequence at: i) toNFA.
		last endNode connectTo: subGraph startNode. last := subGraph
	].
	^ ret endNode: last endNode! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
asDFA
	
	^ self asNFA asDFA! !

!ParsecLexerNode methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ self subclassResponsibility! !

!ParsecLexerNode methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ self subclassResponsibility! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
asNFA
	|ret endNode|
	ret := self toNFA.
	endNode := ParsecNFANode new.
	ret endNode connectTo: endNode.
	ret endNode: endNode.
	^ ret startNode"ensure ending node"! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
asNFAGraph
	|ret endNode|
	ret := self toNFA.
	endNode := ParsecNFANode new.
	ret endNode connectTo: endNode.
	ret endNode: endNode.
	^ ret "ensure ending node"! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	self subclassResponsibility! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
match: aString

	^ self lexMatch: aString readStream! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
matchAtEnd: aString

	|stream|
	stream := aString readStream.
	^ (self lexMatch: stream) and: [ stream atEnd ].! !

!ParsecLexerNode methodsFor: 'testing' stamp: ' 1/8/2017 13:28:27'!
matchesStreamPrefix: s

	^ self lexMatch: s! !

!ParsecLexerNode methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	self subclassResponsibility! !

ParsecLexerNode subclass: #ParsecLexerOperand
	instanceVariableNames: 'operand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

ParsecLexerOperand subclass: #ParsecLexerAny
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerAny methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ {self hash -> '.' -> '.'} asOrderedCollection! !

!ParsecLexerAny methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name  } ! !

!ParsecLexerAny methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	aStream next. "problem here with backtracking ?"
	^ true! !

!ParsecLexerAny methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| op |
	op := ParsecNFANodeAny new. 
	^ ParsecNFAGraph new startNode: op startNode; endNode: op endNode.
	! !

ParsecLexerOperand subclass: #ParsecLexerEscape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerEscape methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ operand == aStream peek
		ifTrue: [ aStream next. true ]
		ifFalse: [ false ]! !

ParsecLexerOperand subclass: #ParsecLexerGroup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerGroup methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ operand asGraph add: (self hash -> '[]' -> operand hash); yourself! !

!ParsecLexerGroup methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand asList } ! !

!ParsecLexerGroup methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ operand lexMatch: aStream "this should probably be an option, including ranges"! !

!ParsecLexerGroup methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	^ operand toNFA "this is normally an option"
	! !

ParsecLexerGroup subclass: #ParsecLexerNotGroup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerNotGroup methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	|curPosition| "this is not so we need to backtrack on success and procceed on failure"
	curPosition := aStream position.
	^ (super lexMatch: aStream)
		ifTrue: [ aStream position: curPosition. false ]
		ifFalse: [ aStream atEnd ifTrue: [^ false]. 
			aStream next . "groups progress by single chars" true ]! !

!ParsecLexerNotGroup methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| graph new|
	graph := super toNFA. 
	new := ParsecNFANodeNot new.
	graph startNode out do: [:n | new of: n ].
	graph startNode: new.
	^ graph! !

!ParsecLexerOperand class methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
new: anAtom

	^ self new with: anAtom! !

!ParsecLexerOperand methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ {self hash -> 'op' -> (operand asString , ' (' , self hash asString , ')') "unique"} asOrderedCollection! !

!ParsecLexerOperand methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand } ! !

!ParsecLexerOperand methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ operand lexMatch: aStream! !

!ParsecLexerOperand methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
operand

	^ operand! !

!ParsecLexerOperand methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| op |
	op := operand toNFA. "can be an NFANode but startNode and endNode are poly"
	^ ParsecNFAGraph new startNode: op startNode; endNode: op endNode.
! !

!ParsecLexerOperand methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: anAtom

	anAtom isString
		ifTrue: [ anAtom size > 1 ifTrue: [ operand := anAtom ] ifFalse: [ operand := anAtom first ] ]	
		ifFalse: [ operand := anAtom ]! !

ParsecLexerOperand subclass: #ParsecLexerOpt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerOpt methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ operand asGraph add: (self hash -> '?' -> operand hash); yourself! !

!ParsecLexerOpt methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand asList } ! !

!ParsecLexerOpt methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	super lexMatch: aStream.
	^ true "always succeeds"! !

!ParsecLexerOpt methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
toNFA

	|graph start end |
	graph := operand toNFA.
	start := ParsecNFANode new.
	end := ParsecNFANode new.
	start connectTo: graph startNode.
	start connectTo: end.
	graph endNode connectTo: end.
	^ ParsecNFAGraph new startNode: start; endNode: end.! !

ParsecLexerOperand subclass: #ParsecLexerPlus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerPlus methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ operand asGraph add: (self hash -> '+' -> operand hash); yourself! !

!ParsecLexerPlus methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand asList } 
! !

!ParsecLexerPlus methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ (super lexMatch: aStream)
			ifTrue: [ [ super lexMatch: aStream ] whileTrue. true ]
			ifFalse: [ false ]
! !

!ParsecLexerPlus methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	|graph|
	graph := super toNFA.
	graph endNode connectTo: graph startNode.
	^ graph
! !

ParsecLexerOperand subclass: #ParsecLexerRange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerRange methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ (aStream peek between: operand key and: operand value)
			ifTrue: [ aStream next. true ]
			ifFalse: [ false ] ! !

!ParsecLexerRange methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| op |
	op := ParsecNFARangeNode new with: operand. 
	^ ParsecNFAGraph new startNode: op startNode; endNode: op endNode.
! !

ParsecLexerOperand subclass: #ParsecLexerRepetition
	instanceVariableNames: 'between'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerRepetition methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ operand asGraph add: (self hash -> '{}' -> operand hash); yourself ! !

!ParsecLexerRepetition methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand asList } ! !

!ParsecLexerRepetition methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
between: aPair

	between := aPair
! !

!ParsecLexerRepetition methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream
	
	|times currentPosition|
	times := 0.
	currentPosition := aStream position.
	[ super lexMatch: aStream ] whileTrue: [ times := times + 1 ].
	^ (times between: between key and: between value)
		ifTrue: [ true ]
		ifFalse: [ aStream position: currentPosition. false ] 
! !

!ParsecLexerRepetition methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	"unroll -- seem like 0 to 5 will not work"

	| last subGraph ret patchExits repetitionLast |
	ret := ParsecNFAGraph new.
	last := operand toNFA.
	patchExits := OrderedCollection new.
	ret startNode: last startNode.
	2 to: between value do: [:i |
		subGraph := operand toNFA.
		last endNode connectTo: subGraph startNode. last := subGraph.
		i >= between key ifTrue: [ patchExits add: subGraph ]. 
		"2 to 5 , then 2, 3, 4 go directly to then end"
	].
	repetitionLast := ParsecNFANode new.
	patchExits do: [ :graph | graph endNode connectTo: repetitionLast ].
	^ ret endNode: repetitionLast.! !

ParsecLexerOperand subclass: #ParsecLexerStar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerStar methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ operand asGraph add: (self hash -> '*' -> operand hash); yourself! !

!ParsecLexerStar methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name . operand asList } ! !

!ParsecLexerStar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	[ super lexMatch: aStream ] whileTrue.
	^ true "always succeeds"! !

!ParsecLexerStar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	|graph start end |
	graph := operand toNFA.
	start := ParsecNFANode new.
	end := ParsecNFANode new.
	start connectTo: graph startNode.
	start connectTo: end.
	graph endNode connectTo: end.
	graph endNode connectTo: graph startNode.
	^ ParsecNFAGraph new startNode: start; endNode: end.! !

ParsecLexerNode subclass: #ParsecLexerOption
	instanceVariableNames: 'options'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecLexerOption methodsFor: 'adding' stamp: ' 1/8/2017 13:28:27'!
add: aLexerNode

	^ options add: aLexerNode! !

!ParsecLexerOption methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asGraph

	
	^ (options collect: [ :option | option asGraph add: self hash -> '|' -> option hash; yourself ]) deepFlatten asOrderedCollection! !

!ParsecLexerOption methodsFor: 'converting' stamp: ' 1/8/2017 13:28:27'!
asList

	^ { self class name. (self options collect: [ :option | option asList  ]) asArray } ! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
first

	^ options first! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
fourth

	^ options fourth! !

!ParsecLexerOption methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	options := OrderedCollection new! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	options do: [ :node | |currentPos|
		currentPos := aStream position.
		(node lexMatch: aStream) 
			ifTrue: [ ^ true ] 
			ifFalse: [ aStream position: currentPos ] "backtrack -- handles any and others"
	]. "first that matches not longer"
	^ false! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
options

	^ options! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
second

	^ options second! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
size

	^ options size! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
third

	^ options third! !

!ParsecLexerOption methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toNFA

	| last subGraph ret|
	ret := ParsecNFAGraph new.
	ret startNode: ParsecNFANode new.
	ret endNode: ParsecNFANode new.
	1 to: self options size do: [:i |
		subGraph := (self options at: i) toNFA.
		ret startNode connectTo: subGraph startNode. 
		subGraph endNode connectTo: ret endNode.
	].
	^ ret! !

Object subclass: #ParsecNFAGraph
	instanceVariableNames: 'startNode endNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecNFAGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
allNodes

	^ startNode allNodesUntil: endNode! !

!ParsecNFAGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
endNode
	^ endNode! !

!ParsecNFAGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
endNode: anObject
	endNode := anObject! !

!ParsecNFAGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
startNode
	^ startNode! !

!ParsecNFAGraph methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
startNode: anObject
	startNode := anObject! !

Object subclass: #ParsecNFANode
	instanceVariableNames: 'char out outWithoutEpsilons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
allNodesUntil: anotherNode
	
	
	^ self allNodesUntil: anotherNode given: {  } asSet. ! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
allNodesUntil: anotherNode given: aSet
	
	(aSet includes: self) ifTrue: [ ^ aSet] ifFalse: [ aSet add: self].
	self == anotherNode
		ifFalse: [ out do: [ :n | n allNodesUntil: anotherNode given: aSet ] ].
	^ aSet! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
asDFA

	"unroll collect and anySatisfy here, after testing"

	|setToInt counter newSetStack finalCollection|
	finalCollection := OrderedCollection new.
	setToInt := Dictionary new.
	newSetStack := FILOStack new.
	counter := 0.
	newSetStack push: (self isAuto ifFalse: [{self}] ifTrue: [self followEpsilons])  asSet. "follow epsilons in the initial state".
	setToInt at: newSetStack top put: (counter := counter + 1).
	[newSetStack isEmpty] whileFalse: [ |current|
		current := newSetStack pop.  
		finalCollection add: (Array new: 257). "the last one marks if this is end"
		finalCollection last at: 257 put: (current anySatisfy: [:n | n isEnd]). 
		0 to: 255 do: [ :i |  |newSet|
			newSet := (current collect: [ :n | n lexMatch: (Character value: i)]) deepFlatten asSet.
			newSet isEmpty
				ifTrue: [ finalCollection last at: i + 1 put: nil ]
			 	ifFalse: [  |index|
					index := setToInt at: newSet ifAbsent: [ newSetStack push: newSet. setToInt at: newSet put: (counter := counter + 1) ].
					finalCollection last at: i + 1 put: index.
				]
		]
	].

	^ ParsecDFAArray new with: finalCollection asArray! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
asGraph

	^ self asGraph: Set new! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
asGraph: aSet

	^ (aSet includes: self) 
		ifTrue: [ {  } asOrderedCollection ]
		ifFalse: [ aSet add: self. (out collect: [ :n | (n asGraph: aSet) add: self hash -> char -> n hash; yourself ]) deepFlatten asOrderedCollection ] ! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
beNot
	char == nil ifFalse: [ 
		|n|
		n := ParsecNFANodeNot new.
		self become: n. 		
		self of: n. "magic"
	] "do not negate epsilons"
	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
char

	^ char

	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
char: aChar

	char := aChar! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
check: aChar

	^ aChar == char	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
connectTo: anotherNode

	out add: anotherNode! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
endNode "polyMorphic with NFAGraph"

	^ self
	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
followEpsilons

	^ outWithoutEpsilons ifNil: [outWithoutEpsilons := (out collect: [ :n | 
			n isAuto 
				ifTrue: [n followEpsilons] "follow epsilons"
				ifFalse: [ n ]
		]) deepFlatten]
								! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	out := Set new.
	char := nil.! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
isAuto

	^ char == nil and: [ out isEmpty not "end" ]
	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
isEnd

	^ out isEmpty
	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aChar "aChar can be nil marking auto"

	^ 	(self check: aChar) 
			ifTrue: [ self followEpsilons ] 
			ifFalse:[ {  } ] 
		! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
match: aString

	^ self matchesStreamPrefix: aString readStream! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
matchesStreamPrefix: s

	"unroll collect and anySatisfy here, after testing"

	|cList nList endPos initPosition possibleZeroMatch |
	possibleZeroMatch := false.
	endPos := initPosition := s position.
	cList := nList := (self isAuto ifFalse: [{self}] ifTrue: [self followEpsilons])  asSet. "follow epsilons in the initial state"
	[ nList isEmpty or: [s atEnd ] ] whileFalse: [ |c|
		c := s peek.
		cList := nList. 
		nList := (cList collect: [ :n | 
			n isEnd ifTrue: [ possibleZeroMatch := true. endPos := s position ]. "mark ends"
			n lexMatch: c.
		]) deepFlatten.
		nList isEmpty ifFalse: [ s next ]
	].
	nList do: [ :n | n isEnd ifTrue: [ possibleZeroMatch := true. endPos := s position] ]. "last iteration"
	s position: endPos.
	^ endPos ~= initPosition or: [ possibleZeroMatch ] ! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
out

	^ out
	! !

!ParsecNFANode methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
startNode "polyMorphic with NFAGraph"

	^ self
	! !

ParsecNFANode subclass: #ParsecNFANodeAny
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecNFANodeAny methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
asGraph: aSet

	^ (aSet includes: self) 
		ifTrue: [ {  } ]
		ifFalse: [ aSet add: self. (out collect: [ :n | n asGraph add: self hash -> '.' -> n hash; yourself ]) deepFlatten asOrderedCollection]! !

!ParsecNFANodeAny methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
check: aChar

	^ true		! !

!ParsecNFANodeAny methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
isAuto

	^ false! !

ParsecNFANode subclass: #ParsecNFANodeNot
	instanceVariableNames: 'nfaNodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecNFANodeNot methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
asGraph: aSet
	
	| nots |
	^ (aSet includes: self) 
		ifTrue: [ {  } asOrderedCollection ]
		ifFalse: [ aSet add: self.
			nots := nfaNodes collect: [ :n | n char ]. 
			(out collect: [ :n | n asGraph add: self hash -> ('^ ' , nots asString) -> n hash; yourself ]) deepFlatten asOrderedCollection]! !

!ParsecNFANodeNot methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
check: aChar

	nfaNodes do: [ :n | (n check: aChar) ifTrue: [^ false]].
	^ true ! !

!ParsecNFANodeNot methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	super initialize.
	nfaNodes := Set new.! !

!ParsecNFANodeNot methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
isAuto

	^ false "since char = nil"! !

!ParsecNFANodeNot methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
of: aNFANode

	nfaNodes add: aNFANode.
	out addAll: aNFANode out! !

ParsecNFANode subclass: #ParsecNFARangeNode
	instanceVariableNames: 'rangePair'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!ParsecNFARangeNode methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
asGraph: aSet

	^ (aSet includes: self) 
		ifTrue: [ {  } asOrderedCollection ]
		ifFalse: [ (out collect: [ :n | n asGraph add: self hash -> rangePair -> n hash; yourself ]) deepFlatten asOrderedCollection]! !

!ParsecNFARangeNode methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
check: aChar

	^ aChar between: rangePair key  and: rangePair value
			! !

!ParsecNFARangeNode methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
isAuto

	^ false! !

!ParsecNFARangeNode methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: aRangePair

	rangePair := aRangePair! !

Object subclass: #ParsecParser
	instanceVariableNames: 'algo grammar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

ParsecParser subclass: #EPEGgrammar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!EPEGgrammar commentStamp: 'NickPapoulias 11/30/2016 15:33' prior: 0!
Bootstrap EPEG syntax (can encapsulate lops and extend with new syntactic ops )!

ParsecParser subclass: #ParsecExprGrammar
	instanceVariableNames: 'start expression number parentheses multiplicationOrDivision additionOrSubstraction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecExprGrammar class methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
log

	^ true! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
additionOrSubstraction

	^ additionOrSubstraction --> 
		expression , '\+|-' , expression ==> [ :tokens | 
			tokens first perform: tokens second value asSymbol withArguments: {tokens third} ].! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
expression

	^ expression --> parentheses
		| multiplicationOrDivision
		| additionOrSubstraction
		| number ! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
multiplicationOrDivision

	^ multiplicationOrDivision --> 
		expression , '\*|/' , expression ==> [ :tokens | 
			tokens first perform: tokens second value asSymbol withArguments: {tokens third} ].! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
number

	^ number --> '[0-9]+(\.[0-9]+)?' ==> [ :token | token value asNumber ]! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
parentheses

	^ parentheses --> '\(' , expression , '\)' ==> [ :tokens | tokens second ].! !

!ParsecExprGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
start

	^ start --> expression ! !

ParsecParser subclass: #ParsecJsonGrammar
	instanceVariableNames: 'members pair string value elements number object array ttrue tfalse null start'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecJsonGrammar methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
array

	^ array --> '\[' , elements opto , ']' ! !

!ParsecJsonGrammar methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
elements

	^ elements --> value , #(',' , value) staro "tokenized as is seperatedBy op can be handy here"! !

!ParsecJsonGrammar methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
members

	^ members --> pair , #(',' , pair) staro! !

!ParsecJsonGrammar methodsFor: 'tokens' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
null

	^ null --> 'null'! !

!ParsecJsonGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
number

	^ number --> ('-?[0-9]+' , '\.[0-9]+' lopt , '[eE][-+]?[0-9]+' lopt) ! !

!ParsecJsonGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
object

	^ object --> '\{' , members opto , '}' ! !

!ParsecJsonGrammar methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
pair

	^ pair --> string , '\:' , value! !

!ParsecJsonGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
start

	^ start --> value! !

!ParsecJsonGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
string

	^ string --> ('"' , ('\\[\\/"bfnrt]' || '\\u[0-9a-fA-F]{4}' || '[^\\"]') lstar , '"') ! !

!ParsecJsonGrammar methodsFor: 'tokens' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
tfalse

	^ tfalse --> 'false' ! !

!ParsecJsonGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
ttrue

	^ ttrue --> 'true' ! !

!ParsecJsonGrammar methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
value

	^ value --> string / number / object / array / ttrue / tfalse / null! !

ParsecJsonGrammar subclass: #ParsecJsonLoggingGrammar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecJsonLoggingGrammar class methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
log

	^ true! !

ParsecJsonGrammar subclass: #ParsecJsonParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

ParsecJsonParser subclass: #ParsecJsonLoggingParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecJsonLoggingParser class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
log

	^ true! !

!ParsecJsonParser methodsFor: 'grammar' stamp: ' 1/8/2017 13:28:27'!
array

	^ array --> '\[' , elements opto , ']' ! !

!ParsecJsonParser methodsFor: 'grammar' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
elements

	^ elements --> value , #(',' , value) staro! !

!ParsecJsonParser methodsFor: 'grammar' stamp: ' 1/8/2017 13:28:27'!
members

	^ members --> pair , #(',' , pair) staro! !

!ParsecJsonParser methodsFor: 'tokens' stamp: ' 1/8/2017 13:28:27'!
null

	^ null --> 'null'! !

!ParsecJsonParser methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
number

	^ number --> ('-?[0-9]+' , '\.[0-9]+' lopt , '[eE][-+]?[0-9]+' lopt) ! !

!ParsecJsonParser methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
object

	^ object --> '\{' , members opto , '}' ! !

!ParsecJsonParser methodsFor: 'grammar' stamp: ' 1/8/2017 13:28:27'!
pair

	^ pair --> string , '\:' , value! !

!ParsecJsonParser methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
start

	^ start --> value! !

!ParsecJsonParser methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
string

	^ string --> ('"' , ('\\[\\/"bfnrt]' || '\\u[0-9a-fA-F]{4}' || '[^\\"]') lstar , '"') ! !

!ParsecJsonParser methodsFor: 'tokens' stamp: ' 1/8/2017 13:28:27'!
tfalse

	^ tfalse --> 'false' ! !

!ParsecJsonParser methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
ttrue

	^ ttrue --> 'true' ! !

!ParsecJsonParser methodsFor: 'grammar' stamp: ' 1/8/2017 13:28:27'!
value

	^ value --> string / number / object / array / ttrue / tfalse / null! !

!ParsecParser class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
algoDefaultClass

	^ ParsecMixedOrderAlgo! !

!ParsecParser class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
log

	^ false! !

!ParsecParser methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
initialize

	super initialize.
	
	self class allInstVarNames 
			select: [ :iv | (ParsecParser instVarNames includes: iv) not ] 
			thenDo: [ :iv | self instVarNamed: iv put: iv ]. "ivs as symbols"
	
	"if we put direct references inside rules it should be done here and then we just
	re-initialize"
	
	grammar := ParsecGrammar 
		new named: self class name; 
		with: ((self class allSelectorsBelow: ParsecParser) collect: 
			[ :name | self perform: name]).! !

!ParsecParser methodsFor: 'parsing' stamp: 'NickPapoulias 1/17/2017 20:52'!
parse: aString
	|algoInst |
	algo ifNil: [  algo := self class algoDefaultClass ].
	algoInst := self class log 
		ifTrue: [ algo new log ]
		ifFalse: [ algo new ].
		
	^ self parse: aString using: algoInst! !

!ParsecParser methodsFor: 'parsing' stamp: 'NickPapoulias 4/12/2017 12:36'!
parse: aString using: algoInst
	|charts|
	charts := (algoInst  parse: aString using: grammar). 

	self halt.

	^ ParsecResult new
		 charts: charts;
		 value: charts successfullTrees! !

!ParsecParser methodsFor: 'instance creation' stamp: ' 1/8/2017 13:28:27'!
with: anAlgorithm

	algo := anAlgorithm ! !

Object subclass: #ParsecResult
	instanceVariableNames: 'charts value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecResult methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 12:52'!
charts
	^ charts! !

!ParsecResult methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 12:52'!
charts: anObject
	charts := anObject! !

!ParsecResult methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 12:53'!
succeeded

	^ charts succeeded! !

!ParsecResult methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 12:52'!
value
	^ value! !

!ParsecResult methodsFor: 'accessing' stamp: 'NickPapoulias 1/19/2017 12:52'!
value: anObject
	value := anObject! !

Object subclass: #ParsecRule
	instanceVariableNames: 'name rightHandSide grammar childRules ordered isAnd isNot iregExpr directRefs firsts follow canBeEmpty predict action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

ParsecRule subclass: #ParsecGammaStart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecGammaStart methodsFor: 'accessing' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
directRefsAt: anIndex

	^ directRefs ifNil: [ directRefs := ((rightHandSide at: anIndex) collect: [ :symbol | grammar at: symbol ])]
	! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
, aSymbolOrStringOrArray


	aSymbolOrStringOrArray isSymbol
		ifTrue: [ rightHandSide last addLast: aSymbolOrStringOrArray ]
		ifFalse: [ childRules addLast: aSymbolOrStringOrArray asRule. self , childRules last name ]
	
	! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
/ aSymbolOrString

	ordered := true.
	self | aSymbolOrString! !

!ParsecRule methodsFor: 'ops' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
==> aBlock

	action := aBlock
	
	! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
\ aSymbolOrString

	ordered := true.
	self | aSymbolOrString! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
action
	^ action! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
action: anObject
	action := anObject! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
and

	isAnd := true! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
asGammaStart

	ParsecGammaStart adoptInstance: self! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
asRule

	^ self
	
	! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
asScannerStateFrom: start to: end withIndex: anAlternative

	^ ParsecState on: self alternative: anAlternative from: start to: end dot: 1 "ie at the end"
	"the alternative is not the correct one here -- but the one return from the terminal match"! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
at: anIndex

	^ rightHandSide at: anIndex! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
beVanilla

	isNot := false.
	isAnd := false.! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
canBeEmpty

	^ canBeEmpty ! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
computeFirsts

	firsts := OrderedCollection new.
	rightHandSide do: [ :rule | 
		firsts add: (self firstForRightHandSide: rule)
	].
	canBeEmpty := firsts deepFlatten anySatisfy: [ :r | r isEmpty ].
	^ firsts := firsts	asParsecCollection.
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
computeFollow

	self isTerminal ifTrue: [ ^ self ].
	
	rightHandSide do: [ :rule | 
		|firstEpsilon lastEpsilon|  
				1 to: rule size do: [ :index | 
					((self flattenFirstsFor: (rule at: index)) anySatisfy: [ :r | r isEmpty ])
						ifTrue: [ firstEpsilon ifNil: [ firstEpsilon := index ]. lastEpsilon := index ]].
		1 to: rule size do: [ :index | 
			|left right|
			left := rule at: index.
			right := rule at: index + 1 ifAbsent: [nil] "last one".
			right ifNotNil: [(self followFor: left) 
				addAll: ((self flattenFirstsFor: right) select: [ :r | ( r isEmpty) not ])].
			((lastEpsilon = rule size and: [ index + 1 >= firstEpsilon ]) or: [ index = rule size ])
				ifTrue: [ (self followFor: left) addAll: self follow ] 
			]
		]			
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/16/2017 14:45'!
"protocol: filtering"
computePredict

	^ predict := self firsts 
		collect: [ :f | 
			(f anySatisfy: [ :r | r isEmpty ])
				ifTrue: [ (f select: [ :r | r isEmpty not ]) union: follow ]
			 	ifFalse: [ f ]
		]
		
	
	! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
directRefsAt: anIndex

	^ self isTerminal 
		ifTrue: [ self ]
		ifFalse: [  
			directRefs at: anIndex "ifAbsent: [
				directRefs 
					at: anIndex 
					put: ((rightHandSide at: anIndex) collect: [ :symbol | grammar at: symbol ]) ]"
	]! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
firstForRightHandSide: rule
	
	|symbolFirsts totalFirsts index|
	
	index := 1.
	totalFirsts := { } asOrderedCollection.
	[symbolFirsts = nil or: [(symbolFirsts includes: #e) and: [ index <= rule size ]]]
		whileTrue: [ 
			symbolFirsts := self firstsFor: (rule at: index).
			totalFirsts addAll: symbolFirsts. "will return asSet at the end" 
			index := index + 1 
		].
	
	^ totalFirsts deepFlatten asSet asOrderedCollection
! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/7/2017 22:51'!
firsts

	"should be precomputed after all rules are in but before parsing so to 
	avoid lazy overhead while parsing, on live update should be recomputed.
	Infinite recursion avoided because of memoizaton currently (recheck)"

	^ firsts ifNil: [ self computeFirsts ]
	
	! !

!ParsecRule methodsFor: 'filtering' stamp: ' 1/8/2017 13:28:27'!
firsts: aList

	"should be precomputed after all rules are in but before parsing so to 
	avoid lazy overhead while parsing, on live update should be recomputed.
	Infinite recursion avoided because of memoizaton currently (recheck)"

	firsts := aList! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
firstsAsStrings


	^ self flattenFirsts collect: [ :f | f isEmptyOrEof ifTrue: [ f name ] ifFalse: [  f regExpr regExpr ] ]
	
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
firstsFor: aSymbol

	^ (self grammar at: aSymbol) firsts. "overriden in e and terminals, check and/not cases"! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/8/2017 16:54'!
flattenFirsts

	"should be precomputed after all rules are in but before parsing so to 
	avoid lazy overhead while parsing, on live update should be recomputed.
	Infinite recursion avoided because of memoizaton currently (recheck)"

	^ self firsts deepFlatten asSet
	
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/8/2017 16:55'!
flattenFirstsFor: aSymbol

	^ (self grammar at: aSymbol) flattenFirsts. "overriden in e and terminals"! !

!ParsecRule methodsFor: 'filtering' stamp: ' 1/10/2017 20:14:21'!
follow

	^ follow 
		ifNil: [ follow := Set new ]
	
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
followAsStrings


	^ self follow collect: [ :f | f isEmptyOrEof ifTrue: [ f name ] ifFalse: [  f regExpr regExpr ] ]
	
	! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/6/2017 16:22'!
followFor: aSymbol

	^ (self grammar at: aSymbol) follow. "overriden in e and terminals"! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
grammar

	^ grammar! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
grammar: aGrammar

	grammar := aGrammar! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
ignoreRegExpr

	^ iregExpr ifNil: [ iregExpr := (self grammar at: #ignore) regExpr ]
	! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
initialize

	super initialize.
	rightHandSide := ParsecCollection new.
	rightHandSide add: ParsecCollection new.
	childRules := ParsecCollection new.
	isAnd := false. 
	isNot := false. 
	ordered := false. "by default -- changes with \"! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
isAnd

	^ isAnd! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
isEmpty

	^ false! !

!ParsecRule methodsFor: 'testing' stamp: 'NickPapoulias 1/17/2017 15:08'!
isEmptyOrEof

	^ false! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
isEof

	^ false! !

!ParsecRule methodsFor: 'testing' stamp: ' 1/8/2017 13:28:27'!
isLexicalGiven: anOperand

	^ self isLex! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
isNot

	^ isNot! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
isOrdered

	^ ordered! !

!ParsecRule methodsFor: 'ops' stamp: 'NickPapoulias 1/17/2017 21:21'!
isPureTerminal

	^ false! !

!ParsecRule methodsFor: 'ops' stamp: 'NickPapoulias 1/19/2017 16:17'!
isSubRule

	^ false! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
isTerminal

	^ false! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
name
	^ name! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
name: anObject
	name := anObject! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
not

	isNot := true! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
opt

	^ { self name . #opt } asRule! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
opto

	^ { self name . #opto } asRule! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
plus
	
	^ { self name . #plus } asRule! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
pluso
	
	^ { self name . #pluso } asRule! !

!ParsecRule methodsFor: 'accessing' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
preComputeDirectRefs
	
	directRefs := Array new: rightHandSide size.
	1 to: rightHandSide size do: [ :i | 
		directRefs 
					at: i 
					put: ((rightHandSide at: i) collect: [ :symbol | symbol isString ifTrue: [ grammar at: symbol ] ifFalse: [ symbol ] ])  ]. 
	"directRefs := directRefs values asOrderedCollection asArray."
	
! !

!ParsecRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/16/2017 14:44'!
predict

	^ predict ifNil: [self computePredict]
		
	! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
rightHandSide

	^ rightHandSide! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
rightHandSideAsNewStatesOn: readingPosition fromTerminal: fromTerminal toTerminal: toTerminal

	| newStates |
	newStates := ParsecCollection new.

	1 to: self rightHandSide size do: [ :i |
		newStates add: 
			((ParsecState 
				on: self alternative: i 
				from: readingPosition 
				to: readingPosition 
				dot: 0)
				fromTerminal: fromTerminal;
				toTerminal: toTerminal)
	].

	^ newStates! !

!ParsecRule methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
rightHandSideSize

	^ rightHandSide size! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
shouldIgnore: aCharacter

	^ { 32 . 10 . 9 . 13 . 12 } includes: aCharacter asciiValue

! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
star
	
	^ { self name . #star } asRule! !

!ParsecRule methodsFor: 'ops' stamp: ' 1/8/2017 13:28:27'!
staro
	
	^ { self name . #staro } asRule! !

!ParsecRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
tempRules

	^ childRules! !

!ParsecRule methodsFor: 'logical operations' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
| aSymbolOrString

	rightHandSide addLast: ParsecCollection new.
	self , aSymbolOrString! !

ParsecRule subclass: #ParsecSubRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecSubRule methodsFor: 'testing' stamp: 'NickPapoulias 1/17/2017 14:24'!
isSubRule

	^ true! !

ParsecRule subclass: #ParsecTerminalRule
	instanceVariableNames: 'regExpr lastInput lastResult lastPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

ParsecTerminalRule subclass: #ParsecEmptyRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecEmptyRule methodsFor: 'filtering' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
computeFirsts

	^ firsts := { {self} asSet } asOrderedCollection! !

!ParsecEmptyRule methodsFor: 'filtering' stamp: 'NickPapoulias 1/8/2017 17:11'!
firsts

	^ firsts ifNil: [ self computeFirsts]! !

!ParsecEmptyRule methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
isEmpty

	^ true! !

!ParsecEmptyRule methodsFor: 'testing' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
isEmptyOrEof

	^ true! !

!ParsecEmptyRule methodsFor: 'testing' stamp: 'NickPapoulias 1/19/2017 16:23'!
isPureTerminal

	^ false! !

!ParsecEmptyRule methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
name

	^ #e! !

ParsecTerminalRule subclass: #ParsecEof
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecEof methodsFor: 'accessing' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
computeFirsts

	^ firsts := { {self} asSet } asOrderedCollection! !

!ParsecEof methodsFor: 'accessing' stamp: 'NickPapoulias 1/8/2017 17:12'!
firsts

	^ firsts ifNil: [self computeFirsts]! !

!ParsecEof methodsFor: 'testing' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
isEmptyOrEof

	^ true! !

!ParsecEof methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
isEof

	^ true! !

!ParsecEof methodsFor: 'testing' stamp: 'NickPapoulias 1/19/2017 16:23'!
isPureTerminal

	^ false! !

!ParsecEof methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
name

	^ #Eof! !

!ParsecTerminalRule methodsFor: 'ops' stamp: ' 1/10/2017 20:14:21'!
, aString

	rightHandSide last addLast: (aString asParsecRegex asDFA regExpr: aString)! !

!ParsecTerminalRule methodsFor: 'ops' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
asString

	^ self regExpr regExpr! !

!ParsecTerminalRule methodsFor: 'accessing' stamp: 'Nick Papoylias 1/10/2017 21:18:48'!
computeFirsts

	^ firsts := { {self} asSet } asOrderedCollection! !

!ParsecTerminalRule methodsFor: 'accessing' stamp: 'NickPapoulias 1/8/2017 17:11'!
firsts

	^ firsts ifNil: [self computeFirsts]! !

!ParsecTerminalRule methodsFor: 'testing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
isEmptyOrEof

	^ false! !

!ParsecTerminalRule methodsFor: 'testing' stamp: 'NickPapoulias 1/17/2017 21:09'!
isPureTerminal

	^ true! !

!ParsecTerminalRule methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
isTerminal

	^ true! !

!ParsecTerminalRule methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
regExpr

	^ regExpr ifNil: [regExpr := self rightHandSide first first]! !

!ParsecTerminalRule methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/16/2017 16:33:48'!
terminalMatchForInput: input atPosition: readingPosition
	
	(readingPosition == lastPosition and: [ lastInput == input ]) ifTrue: [^ lastResult]. "last position memoize -- threaded ?"
	lastInput := input. lastPosition := readingPosition.
	readingPosition == input size 
		ifTrue: [ ^ lastResult := readingPosition  ]. "no match already at end"		
	input directSetPos: readingPosition. "already the case ?"
	^ lastResult := ((self regExpr matchesStreamPrefix: input)
			ifTrue: [self ignoreRegExpr matchesStreamPrefix: input. input position]
			ifFalse: [ readingPosition ])
			! !

ParsecTerminalRule subclass: #ParsecTerminalSubRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecTerminalSubRule methodsFor: 'testing' stamp: 'NickPapoulias 1/17/2017 21:09'!
isPureTerminal

	^ true! !

Object subclass: #ParsecState
	instanceVariableNames: 'rule dotPosition from to fromTerminal toTerminal rightHandIndex ruleAtDot rightHand rightHandSize dRefRightHand childrenStates childrenForest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecState class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
on: rule alternative: index from: start to: finish dot: dotPosition

	^ self new
		on: rule
		alternative: index
		from: start
		to: finish
		dot: dotPosition
		! !

!ParsecState methodsFor: 'action' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
action

	^ rule action

! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
advanceDot

	dotPosition := dotPosition + 1.
	ruleAtDot := nil

! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
asScannerStateFrom: start to: end withIndex: anAlternative


	^ (self ruleAtDot asScannerStateFrom: start to: end withIndex: anAlternative)
			fromTerminal: self toTerminal;
			toTerminal: self toTerminal + 1
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 1/16/2017 12:13'!
beginsWithNot

	^ dRefRightHand first isNot

! !

!ParsecState methodsFor: 'action' stamp: 'NickPapoulias 3/25/2017 16:44'!
childrenForest

	^ childrenForest ifNil: [childrenForest := self 
		populate: { {  } asOrderedCollection } asOrderedCollection 
		with: (childrenStates at: self from ifAbsent: [ ^ { {  } asOrderedCollection } asOrderedCollection  ])].

! !

!ParsecState methodsFor: 'action' stamp: 'NickPapoulias 3/23/2017 20:17'!
childrenForest: forest

	childrenForest := forest
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
childrenStates

	^ childrenStates! !

!ParsecState methodsFor: 'accessing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
childrenStates: aCopy

	childrenStates := aCopy! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
childrenWithPositions

	^ childrenStates collect: [ :child | self -> (child from -> child to) -> child ]! !

!ParsecState methodsFor: 'action' stamp: 'np 3/28/2017 17:07'!
copy

	|clone|
	clone := super copy.
	clone childrenStates:  childrenStates copy.
	^ clone.
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
dotPosition

	^ dotPosition! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
end: anIndex

	to := anIndex! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
equalTo: anotherState

	^ rule == anotherState rule
	and: [rightHandIndex == anotherState rightHandIndex 
	and: [dotPosition == anotherState dotPosition
	and: [ from == anotherState from 
	and: [ to == anotherState to ]]]]! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
expecting: name at: start
	^ to == start
		and: [ self isIncomplete and: [ self ruleAtDot name == name ]]
			! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
expectingAnd

	^ self ruleAtDot isAnd! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
expectingEmpty

	^ self ruleAtDot isEmpty! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
expectingEof

	^ self ruleAtDot isEof! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
expectingNot

	^ self ruleAtDot isNot! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
expectingTerminal

	^ self ruleAtDot isTerminal! !

!ParsecState methodsFor: 'filtering' stamp: 'NickPapoulias 1/16/2017 12:03'!
firsts

	^ rule firsts at: rightHandIndex
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
from

	^ from
	! !

!ParsecState methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
fromTerminal
	^ fromTerminal! !

!ParsecState methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
fromTerminal: anObject
	fromTerminal := anObject! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 3/23/2017 13:47'!
initialize

	childrenStates := Dictionary new.
	fromTerminal := toTerminal := 0.! !

!ParsecState methodsFor: 'testing' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
isComplete

	^ dotPosition >= rightHandSize! !

!ParsecState methodsFor: 'testing' stamp: ' 1/8/2017 13:28:27'!
isFinal

	^ rule name == #gamma and: [ self isIncomplete not ]! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
isIncomplete

	^ dotPosition < rightHandSize! !

!ParsecState methodsFor: 'action' stamp: 'NickPapoulias 1/19/2017 16:17'!
isSubRule

	^ rule isSubRule

! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
nameAtDot

	self isIncomplete 
		ifTrue: [ rightHand at: dotPosition + 1 ]
		ifFalse: [ nil ]
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
on: aRule alternative: index from: start to: finish dot: aDotPosition

	rule := aRule.
	from := start.
	to := finish.
	dotPosition := aDotPosition.
	rightHandIndex := index.
	rightHand := rule at: rightHandIndex.
	rule isTerminal ifFalse: [dRefRightHand := (rule directRefsAt: rightHandIndex)].
	rightHandSize := rightHand size.! !

!ParsecState methodsFor: 'action' stamp: 'NickPapoulias 3/25/2017 16:55'!
populate: aForest with: childOrChildren 

	childOrChildren isCollection 
		ifFalse: [ 
			childOrChildren rule isTerminal ifFalse: [ .
				childOrChildren childrenForest size > 1 ifTrue: [
					^ self 
						populate: aForest 
						with: (childOrChildren childrenForest collect: [ :tree | 
							childOrChildren shallowCopy childrenForest: { tree } asOrderedCollection ])
				]. "recurse in depth"
			].
			aForest do: [ :tree | tree add: childOrChildren ].
			childOrChildren rule isEof ifTrue: [ ^ aForest ].
			self populate: aForest with: (childrenStates at: childOrChildren to ifAbsent: [^ aForest]).
		]
		ifTrue: [ 
			|newForests| 
			newForests := childOrChildren allButFirst
				 collect: [ :child | 
					self 
						populate: (aForest collect: [:tree | tree copy]) 
						with: child].
			self populate: aForest with: childOrChildren first.
			newForests do: [:forest | aForest addAll: forest].
		].
	
	^ aForest! !

!ParsecState methodsFor: 'as yet unclassified' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
previousStates

	^ childrenStates! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
printOn: aStream

	aStream nextPutAll: rule name; nextPutAll: ' --> '.
	
	rightHand doWithIndex: [:item :index |
		dotPosition + 1 = index ifTrue: [ aStream nextPutAll: ' . ' ].
		aStream nextPutAll: item asString; nextPutAll: ' '.
	].

	aStream nextPutAll: ' [ '; nextPutAll: (from->to) asString; nextPutAll: ' ] '.	
	aStream nextPutAll: ' [ '; nextPutAll: (fromTerminal->toTerminal) asString; nextPutAll: ' ] '.	
! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
rightHandIndex

	^ rightHandIndex! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
rightHandSideAsNewStatesOn: readingPosition

	^ self ruleAtDot 
		rightHandSideAsNewStatesOn: self to 
		fromTerminal: self toTerminal 
		toTerminal: self toTerminal.
			! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
rule

	^ rule! !

!ParsecState methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
ruleAtDot

	^ ruleAtDot ifNil: [  
		self isIncomplete 
			ifTrue: [ ruleAtDot := dRefRightHand at: dotPosition + 1 ]
			ifFalse: [ nil ]
		]! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
start: anIndex

	from := anIndex! !

!ParsecState methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
to

	^ to
	! !

!ParsecState methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toTerminal
	^ toTerminal! !

!ParsecState methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
toTerminal: anObject
	toTerminal := anObject! !

!ParsecState methodsFor: 'action' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
value: input withRoot: aRoot peeking: alternatives

	"optimize by treating alternatives as a special case 
	(ie flag the presense of alternatives and recompute values approprietly)"

	| tokens newAlternatives |
	
	newAlternatives := alternatives.
	
	(alternativeSelf isNotNil and: [ alternatives isNotNil ]) ifTrue: [ 
		(alternativeSelf difference: alternatives) 
			do: [:alternative | aRoot valueRoot: input withAlternatives: (alternatives copy add: alternative; yourself). newAlternatives := nil. ].
		(alternatives intersection: self alternativeSelf) 
			do: [ :alternative |  ^ alternative value: input withRoot: aRoot peeking: nil ].	
	]. 
		"nil flags that a parent alternative has alrady been taken"
			
	self rule isPureTerminal
		ifTrue: [ tokens := ParsecToken new with: self for: input ]
		ifFalse: [ 
			tokens := ParsecCollection new.
			1 to: childrenStates size do: [ :i | 
				| s |
				s := childrenStates at: i. 
				s rule isEmptyOrEof
					ifFalse: [ tokens add: (s value: input withRoot: aRoot peeking: newAlternatives) ] ].
			tokens size == 1 ifTrue: [ tokens := tokens first ].
		].
	^ self action
		ifNil: [ tokens ]
		ifNotNil: [ self action value: tokens ]! !

!ParsecState methodsFor: 'action' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
valueRoot: input withAlternatives: alternatives

	^ values 
		add: (self value: input withRoot: self peeking: alternatives); yourself! !

Object subclass: #ParsecToken
	instanceVariableNames: 'parsecState input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecToken methodsFor: 'instance creation' stamp: 'NickPapoulias 1/19/2017 15:28'!
for: aString

	input := aString! !

!ParsecToken methodsFor: 'instance creation' stamp: 'NickPapoulias 1/19/2017 15:53'!
printOn: aStream

	aStream 
		nextPutAll: self printString"; 
		nextPutAll: ' @ ('; 
		nextPutAll: (parsecState from + 1) asString; 
		nextPutAll: ',';
		nextPutAll: (parsecState to) asString;
		nextPutAll: ')'"! !

!ParsecToken methodsFor: 'instance creation' stamp: 'NickPapoulias 1/19/2017 15:40'!
printString

	^ input atAll: (parsecState from + 1 to: parsecState to)! !

!ParsecToken methodsFor: 'instance creation' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
value

	^ input atAll: (parsecState from + 1 to: parsecState to)! !

!ParsecToken methodsFor: 'instance creation' stamp: 'NickPapoulias 1/19/2017 15:04'!
with: aState

	parsecState := aState! !

!ParsecToken methodsFor: 'instance creation' stamp: 'NickPapoulias 1/19/2017 15:29'!
with: aState for: aString

	parsecState := aState.
	input := aString! !

!PositionableStream methodsFor: '*Parsec' stamp: 'Nick Papoylias 1/16/2017 16:33:48'!
directSetPos: aPosition

	position := aPosition! !

!PositionableStream methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
matchPeek: subCollection 
	
	|currentPosition|
	currentPosition := self position.
	1 to: subCollection size do: [:i | (subCollection at: i) == self next ifFalse: [ self position: currentPosition. ^ false ]  ].
	^ true
	! !

!ByteString methodsFor: '*Parsec' stamp: ' 1/10/2017 20:14:21'!
asParsecRegex

 ^ ParsecLexer new parse: self! !

!ByteString methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
asTotRegex

 ^ ParsecLexer new parse: self! !

!ByteString methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
toNFA

	|lastNode firstNode|

	self do: [ :c |  
		lastNode 
			ifNil: [ firstNode := lastNode := c toNFA ]
			ifNotNil: [ |newNode| lastNode connectTo: (newNode := c toNFA). lastNode := newNode ]	
	].

	^ ParsecNFAGraph new startNode: firstNode; endNode: lastNode! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
* number

	|wr|
	
	wr := WriteStream  on: String new.

	1 to: number do: [:i | wr nextPutAll: self].
	
	^ wr contents! !

!String methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
and

	|name|
	name := ('ta' , self hash asString) asSymbol.
	^  (name --|>  self) and. "returns rule"! !

!String methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 15:59'!
asRule

	^ (ParsecTerminalSubRule new name: ('t' , self hash asString) asSymbol) , self! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ aStream matchPeek: self! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
lopt

 ^ '(' , self , ')?'! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
lplus

 ^ '(' , self , ')+'! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
lstar

 ^ '(' , self , ')*'! !

!String methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
not
	
	|name|
	name := ('tn' , self hash asString) asSymbol.
	^  (name --|>  self) not. "returns rule"! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opt

	"A opt = #(A | e)"
	
	^ { self . #| . #e }! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opto

	"A opt = #(A | e)"
	
	^ { self . #\ . #e }! !

!String methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
plus

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #| . #e} . "returns rule"! !

!String methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
pluso

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #\ . #e} . "returns rule"! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
star

	"A* = #(A+ | e)"
	
	^ { self plus . #| . #e }! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
staro

	"A* = #(A+ | e)"
	
	^ { self pluso . #\ . #e }! !

!String methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
|| anotherString "lexical or -- add more like these -- seperate lexical protocol or put in class"

	^ self , '|' , anotherString! !

!Character methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
lexMatch: aStream

	^ aStream peek == self ifTrue: [ aStream next. true ] ifFalse: [ false ]! !

!Character methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
toNFA

	^ ParsecNFANode new char: self! !

IdentityDictionary subclass: #ParsecGrammar
	instanceVariableNames: 'name forked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
binaryOps

	^ #(, | \)! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
contextSensitive

	"add numbered plus ?"

	^ self new 
	named: #Cs;
	with: {  
		#start --> #(A , 'c') and, 'a' pluso , #B.
		#A --> 'a' , #A opto , 'b'.
		#B --> 'b' , #B opto , 'c'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
contextSensitiveWithNot

	"add numbered plus ?"

	^ self new 
	named: #Cs;
	with: {  
		#start --> #(A , 'c') and, 'a' pluso , #B , #('d' not \ 'd' pluso).
		#A --> 'a' , #A opto , 'b'.
		#B --> 'b' , #B opto , 'c'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
english

	"TODO: add #(0-9) for digit , #number and #(A-Za-z) for chars and ids 
	as primitives / terminals, then epsilon, plus, star and ochoice..
	You can have the same with a Class. This is the bigger grammar from 2nd edition.
	
	Optimz Note: the notion of terminal can be generilized to arbitary complex hand-written
	or not expressions"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #Aux , #NP , #VP | #VP.
		#NP --> #Det , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #Nominal , #Noun | #Nominal , #PP.
		#PP --> #Prep, #NP.	
		#VP --> #Verb | #Verb , #NP | #Verb , #NP , #PP | #Verb , #PP | #VP , #PP.
		#Det --> 'that' | 'this' | 'a'.
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' | 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithNewRuleSyntax


	^ self new 
	named: #EnglishGrammar;
	with: #(  
		start -> NP  VP | Aux NP  VP | VP.
		...
	)! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithTemps

	"TODO: add #(0-9) for digit , #number and #(A-Za-z) for chars and ids 
	as primitives / terminals, then epsilon, plus, star and ochoice..
	You can have the same with a Class. This is the bigger grammar from 2nd edition.
	
	Optimz Note: the notion of terminal can be generilized to arbitary complex hand-written
	or not expressions"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #(#Aux , #NP , #VP) | #VP.
		#NP --> #Det , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #(#Nominal , #Noun | #Nominal , #PP).
		#PP --> #Prep, #NP.	
		#VP --> #Verb | #Verb , #NP | #Verb , #NP , #PP | #Verb , #PP | #VP , #PP.
		#Det --> 'that' | 'this' | 'a'.
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' | 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithTempsEmpty

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #Aux , #NP , #VP | #VP.
		#NP --> #Det , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #Nominal , #Noun | #Nominal , #PP.
		#PP --> #Prep, #NP.	
		#VP --> #Verb , #(NP | e) , #(PP | e) | #VP , #PP. "this is actually the op = ?"
		#Det --> 'that' | 'this' | 'a'.
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' | 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithTempsEmptyAndInlineTerminals

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #Aux , #NP , #VP | #VP.
		#NP --> #('that' | 'this' | 'a') , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #Nominal , #Noun | #Nominal , #PP.
		#PP --> #Prep, #NP.	
		#VP --> #Verb , #(NP | e) , #(PP | e) | #VP , #PP. "this is actually the op = ?"
		"#Det --> 'that' | 'this' | 'a'."
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' | 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithTempsEmptyInlineTerminalsAndOps

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #Aux , #NP , #VP | #VP.
		#NP --> #('that' | 'this' | 'a') , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #Nominal , #Noun | #Nominal , #PP.
		#PP --> #Prep, #NP.	
		#VP --> #(Verb , NP opt , PP star) plus. "covers all"
		"#Det --> 'that' | 'this' | 'a'."
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' | 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
englishWithTempsEmptyInlineTerminalsAndOpsMixed

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #NP , #VP | #Aux , #NP , #VP | #VP.
		#NP --> #('that' \ 'this' | 'a') , #Nominal | #ProperNoun | #Pronoun.
		#Nominal --> #Noun | #Nominal , #Noun | #Nominal , #PP.
		#PP --> #Prep, #NP.	
		#VP --> #(Verb , NP opt , PP star) pluso. "covers all"
		"#Det --> 'that' | 'this' | 'a'."
		#Noun --> 'book' | 'flight' | 'meal' | 'money'.
		#Verb --> 'book' | 'include' \ 'prefer'.
		#Aux --> 'does'.
		#Prep --> 'from' | 'to' | 'on' | 'near' | 'through'.
		#ProperNoun --> 'Houston' | 'TWA'.
		#Pronoun --> 'I' | 'She' | 'Me'.
	}! !

!ParsecGrammar class methodsFor: 'accessing' stamp: ' 1/10/2017 20:14:21'!
firstFollowExample

	"add numbered plus ?"

	^ self new 
		named: #FFExample;
		with: {  
			#start --> #E.
			#E --> #T , #E_.
			#E_ --> '\+' , #T , #E_ | #e.
			#T --> #F , #T_.
			#T_ --> '\*' , #F , #T_ | #e.
			#F --> '\(' , #E , '\)' | '[a-zA-Z][a-zA-Z0-9]*'
		}.! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
nonPeg

	"add numbered plus ?"

	^ self new 
	named: #nonPeg;
	with: {  
		#start --> #S.
		#S --> 'x' , #S , 'x' | 'x'
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
nonPegOrd

	"add numbered plus ?"

	^ self new 
	named: #nonPegOrd;
	with: {  
		#start --> #S.
		#S --> 'x' , #S , 'x' \ 'x'
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
number

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #Number.
		#Number --> #Digit plus.
		#Digit --> '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
numberFloat

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #Number , #('\.' , Number) opt.
		#Number --> #Digit plus.
		#Digit --> '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
numberFloatOrdered

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #Number , #('\.' , Number) opto.
		#Number --> #Digit pluso.
		#Digit --> '0' | '1' | '2' | '3' | '4' | '5' | '6' \'7' | '8' | '9'. "one oredered will change the whole rule"
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
numberFloatRegEx

	"See VP on Verb and Debug"

	^ self new 
	named: #EnglishGrammar;
	with: {  
		#start --> #Number , #('\.' , Number) opt.
		#Number --> '[0-9]+'.
	}! !

!ParsecGrammar class methodsFor: 'as yet unclassified' stamp: ' 1/8/2017 13:28:27'!
unaryOps

	^ #(star plus opt staro pluso opto and not) ! !

!ParsecGrammar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
at: aSymbol put: aRule

	aRule grammar: self.
	super at: aSymbol put: aRule! !

!ParsecGrammar methodsFor: 'initialization' stamp: ' 1/8/2017 13:28:27'!
init

	self at: #e put: (ParsecEmptyRule new grammar: self).
	self at: #Eof put: (ParsecEof new grammar: self).
	self at: #ignore put: ((ParsecTerminalRule new grammar: self) , '[ \t\n\r]*'). "can be overriden"! !

!ParsecGrammar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
named

	^ name! !

!ParsecGrammar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
named: aName

	name := aName.
	self init. "subclass of Dict"! !

!ParsecGrammar methodsFor: 'accessing' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
with: aRuleCollection

	|size|
	self withRec: aRuleCollection.
	self values do: [ :v | v preComputeDirectRefs ].
	size := 0. "init first and follow as sets"
	[size ~= (size := self values sum: [:rule |
		"self halt."
		rule computeFirsts. 
		rule flattenFirsts size])] whileTrue. 
	"ie run as long as there is nothing to add -- recheck how this works with memoization"
	size := 0.
	(self at: #start) follow addAll: (self at: #Eof) flattenFirsts.
	[size ~= (size := self values sum: [:rule |
		"self halt."
		rule computeFollow. 
		rule follow size])] whileTrue.
	self values do: [ :v | v computePredict ].
	"lazy compute firsts now
	that all rules are in place and before parsing"! !

!ParsecGrammar methodsFor: 'accessing' stamp: ' 1/8/2017 13:28:27'!
withRec: aRuleCollection

	aRuleCollection do: [ :rule | 
		self at: rule name put: (rule grammar: self).
		self withRec: rule tempRules "if any"
	].
! !

!Symbol methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
--> aSymbolOrString

	^ (ParsecRule new name: self) , aSymbolOrString! !

!Symbol methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:48'!
--|> aSymbolOrString

	^ (ParsecSubRule new name: self) , aSymbolOrString! !

!Symbol methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
and

	|name|
	name := ('ta' , self hash asString) asSymbol.
	^  (name --|>  self) and. "returns rule"! !

!Symbol methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
not
	
	|name|
	name := ('tn' , self hash asString) asSymbol.
	^  (name --|> self) not. "returns rule"! !

!Symbol methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opt

	"A opt = #(A | e)"
	
	^ { self . #| . #e }! !

!Symbol methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opto

	"A opt = #(A | e)"
	
	^ { self . #\ . #e }! !

!Symbol methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
plus

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #| . #e} . "returns rule"! !

!Symbol methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:49'!
pluso

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #\ . #e} . "returns rule"! !

!Symbol methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
star

	"A* = #(A+ | e)"
	
	^ { self plus . #| . #e }! !

!Symbol methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
staro

	"A* = #(A+ | e)"
	
	^ { self pluso . #\ . #e }! !

!OrderedCollection methodsFor: '*Parsec' stamp: 'Nick Papoylias 1/15/2017 17:45:30'!
asParsecCollection

	|pCollection|
	pCollection := ParsecCollection new.
	self do: [ :item | pCollection add: item ].
	^ pCollection! !

TestCase subclass: #ParsecAlgosTest
	instanceVariableNames: 'charts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecAlgosTest commentStamp: 'NickPapoulias 11/5/2016 22:04' prior: 0!
add more assertions!

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testBigJson

	| tJSParser  |
	tJSParser := ParsecJsonGrammar new. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '[
  {
    "_id": "5829933f45655c137f72afb4",
    "index": 0,
    "guid": "1de3c7fa-9f5f-4ce2-bd00-1492485feea5",
    "isActive": true,
    "balance": "$3,545.59",
    "picture": "http://placehold.it/32x32",
    "age": 26,
    "eyeColor": "green",
    "name": {
      "first": "Reeves",
      "last": "Stanton"
    },
    "company": "CHILLIUM",
    "email": "reeves.stanton@chillium.io",
    "phone": "+1 (993) 405-2896",
    "address": "114 Lombardy Street, Tivoli, New York, 5382",
    "about": "Nulla eiusmod sunt cupidatat officia laborum exercitation duis aliquip Lorem laboris nostrud veniam. Sunt in magna consequat reprehenderit exercitation veniam ut consequat exercitation. Occaecat nisi aliquip ad exercitation dolore. Id laboris pariatur aliquip irure ut duis et sunt dolore dolore. Exercitation non aliqua excepteur sint magna laborum. Tempor velit quis ad in Lorem Lorem amet mollit voluptate duis reprehenderit sunt dolor. Dolor qui enim qui mollit labore sunt fugiat.",
    "registered": "Saturday, November 28, 2015 10:53 PM",
    "latitude": "-73.020167",
    "longitude": "134.786528",
    "tags": [
      "adipisicing",
      "irure",
      "velit",
      "in",
      "pariatur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Mccarthy Mcguire"
      },
      {
        "id": 1,
        "name": "Sadie Swanson"
      },
      {
        "id": 2,
        "name": "Hodge Moody"
      }
    ],
    "greeting": "Hello, Reeves!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f561d35c3308fbcd8",
    "index": 1,
    "guid": "f2601a6c-4999-44fb-aeb2-af7849905ca4",
    "isActive": false,
    "balance": "$2,760.73",
    "picture": "http://placehold.it/32x32",
    "age": 30,
    "eyeColor": "brown",
    "name": {
      "first": "Patti",
      "last": "Aguirre"
    },
    "company": "HYDROCOM",
    "email": "patti.aguirre@hydrocom.ca",
    "phone": "+1 (999) 589-2632",
    "address": "837 Adler Place, Brazos, New Mexico, 6246",
    "about": "Magna dolore in nulla esse sint occaecat excepteur deserunt culpa excepteur. Anim quis minim fugiat fugiat anim laborum. Pariatur culpa non id aliqua eu anim sint ad adipisicing tempor non adipisicing laboris. Dolore adipisicing pariatur sit occaecat ut. Aliqua dolor minim excepteur laborum cupidatat nisi est cupidatat ut exercitation. Incididunt est commodo esse ea sint reprehenderit est labore laboris ex minim proident Lorem do.",
    "registered": "Wednesday, October 19, 2016 5:22 PM",
    "latitude": "38.995324",
    "longitude": "-95.68088",
    "tags": [
      "sint",
      "amet",
      "sunt",
      "consectetur",
      "Lorem"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Briggs Chapman"
      },
      {
        "id": 1,
        "name": "Kelley Neal"
      },
      {
        "id": 2,
        "name": "Chandra William"
      }
    ],
    "greeting": "Hello, Patti!! You have 6 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fc55451f6efea29e2",
    "index": 2,
    "guid": "9eed2015-63f5-469b-bc4d-3f625bb742d0",
    "isActive": false,
    "balance": "$2,077.11",
    "picture": "http://placehold.it/32x32",
    "age": 31,
    "eyeColor": "blue",
    "name": {
      "first": "Clemons",
      "last": "Jarvis"
    },
    "company": "COMTRAK",
    "email": "clemons.jarvis@comtrak.org",
    "phone": "+1 (910) 404-2741",
    "address": "258 Dakota Place, Summerset, South Carolina, 2662",
    "about": "Cillum magna tempor voluptate consectetur ullamco enim mollit ullamco aliquip incididunt nisi cupidatat. Nostrud aute est nisi incididunt cupidatat ut dolor ad non nostrud. Dolore labore fugiat consequat consectetur incididunt cupidatat minim laboris. Ullamco est est incididunt nulla nulla minim enim occaecat fugiat ut tempor. Deserunt cupidatat nisi adipisicing est velit id amet qui. Commodo aliqua tempor occaecat ea consectetur consectetur.",
    "registered": "Monday, December 1, 2014 7:42 AM",
    "latitude": "22.819718",
    "longitude": "-159.31187",
    "tags": [
      "deserunt",
      "incididunt",
      "ipsum",
      "enim",
      "velit"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Little Rodgers"
      },
      {
        "id": 1,
        "name": "Sofia Cooley"
      },
      {
        "id": 2,
        "name": "Monroe Jennings"
      }
    ],
    "greeting": "Hello, Clemons!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f376548679c1f7fe3",
    "index": 3,
    "guid": "6e22aeee-d6ee-433f-977c-7bd1acd10273",
    "isActive": false,
    "balance": "$1,811.95",
    "picture": "http://placehold.it/32x32",
    "age": 32,
    "eyeColor": "green",
    "name": {
      "first": "Mathis",
      "last": "Leach"
    },
    "company": "COMTREK",
    "email": "mathis.leach@comtrek.me",
    "phone": "+1 (941) 469-2252",
    "address": "885 Pooles Lane, Morriston, Indiana, 1144",
    "about": "Anim voluptate id nulla officia irure. Anim sunt qui elit ipsum consequat nisi. Minim laborum eiusmod officia nostrud. Duis irure laborum sit deserunt nostrud occaecat pariatur ut consectetur incididunt quis tempor consequat aliquip. Eu excepteur qui fugiat voluptate sint pariatur aliqua commodo ea fugiat aliquip ipsum nulla. Aliquip enim anim commodo pariatur dolor eiusmod fugiat culpa aliquip labore esse officia labore.",
    "registered": "Tuesday, July 15, 2014 12:27 AM",
    "latitude": "-86.086873",
    "longitude": "-145.063901",
    "tags": [
      "aliqua",
      "nisi",
      "cupidatat",
      "voluptate",
      "enim"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Nixon Underwood"
      },
      {
        "id": 1,
        "name": "Sandoval Oneil"
      },
      {
        "id": 2,
        "name": "Jodie Bass"
      }
    ],
    "greeting": "Hello, Mathis!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f91080e43f2815a8f",
    "index": 4,
    "guid": "a110cbcd-da9c-44c5-9740-12795daa38a1",
    "isActive": true,
    "balance": "$3,948.74",
    "picture": "http://placehold.it/32x32",
    "age": 38,
    "eyeColor": "brown",
    "name": {
      "first": "Hardy",
      "last": "Pace"
    },
    "company": "EMOLTRA",
    "email": "hardy.pace@emoltra.biz",
    "phone": "+1 (823) 523-3553",
    "address": "150 Varick Street, Crawfordsville, Rhode Island, 1521",
    "about": "Amet sunt occaecat officia laboris exercitation. Id in amet nostrud eiusmod esse elit. Ullamco laborum occaecat ullamco ipsum qui dolore est pariatur mollit magna nostrud dolore cillum. Excepteur dolore ad consectetur magna pariatur eiusmod. Veniam exercitation proident deserunt magna. Ad consequat deserunt reprehenderit commodo nulla adipisicing id pariatur consequat exercitation do ullamco proident dolore.",
    "registered": "Saturday, February 20, 2016 3:42 AM",
    "latitude": "75.28317",
    "longitude": "176.558437",
    "tags": [
      "ex",
      "quis",
      "ex",
      "qui",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Macdonald Cleveland"
      },
      {
        "id": 1,
        "name": "Caldwell Brewer"
      },
      {
        "id": 2,
        "name": "Justine Beasley"
      }
    ],
    "greeting": "Hello, Hardy!! You have 9 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fe9ef1f88060d2e73",
    "index": 5,
    "guid": "5e034438-ae3a-4981-bc30-c07ccb4ba2c2",
    "isActive": false,
    "balance": "$3,161.10",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "blue",
    "name": {
      "first": "Martha",
      "last": "Spence"
    },
    "company": "COLLAIRE",
    "email": "martha.spence@collaire.net",
    "phone": "+1 (894) 513-2129",
    "address": "788 Crooke Avenue, Gardiner, Connecticut, 4632",
    "about": "Cupidatat fugiat id sint deserunt in et et aliquip qui velit do reprehenderit occaecat et. Nulla nostrud enim id anim irure elit enim fugiat nostrud voluptate est consequat. Voluptate laboris labore occaecat aliquip dolor. Voluptate ad fugiat laboris quis aute deserunt in aliqua.",
    "registered": "Wednesday, August 6, 2014 11:53 AM",
    "latitude": "21.811418",
    "longitude": "71.300384",
    "tags": [
      "incididunt",
      "pariatur",
      "cupidatat",
      "ea",
      "veniam"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Brown Williams"
      },
      {
        "id": 1,
        "name": "Deanna Ferguson"
      },
      {
        "id": 2,
        "name": "Violet Shaw"
      }
    ],
    "greeting": "Hello, Martha!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fc77a2bae9dc5145a",
    "index": 6,
    "guid": "e33f729a-d690-49ba-bc89-ca56f934e0ea",
    "isActive": false,
    "balance": "$3,451.86",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "brown",
    "name": {
      "first": "Stacy",
      "last": "Houston"
    },
    "company": "INSURON",
    "email": "stacy.houston@insuron.com",
    "phone": "+1 (854) 435-3498",
    "address": "959 Concord Street, Aurora, New Hampshire, 7772",
    "about": "Velit exercitation excepteur eiusmod exercitation aliqua duis cupidatat eiusmod aliquip pariatur laborum amet. Laborum adipisicing officia culpa enim pariatur fugiat id officia magna. Sit occaecat Lorem aliquip sint id laborum magna. Consectetur labore laborum reprehenderit magna nostrud adipisicing. Lorem deserunt Lorem sunt dolor. Dolore amet ut commodo nulla Lorem incididunt Lorem sunt labore laborum nisi consectetur.",
    "registered": "Tuesday, November 24, 2015 11:38 AM",
    "latitude": "77.866637",
    "longitude": "-57.468626",
    "tags": [
      "non",
      "esse",
      "velit",
      "deserunt",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Kimberley Mayer"
      },
      {
        "id": 1,
        "name": "Hampton Joyner"
      },
      {
        "id": 2,
        "name": "Bowen Freeman"
      }
    ],
    "greeting": "Hello, Stacy!! You have 9 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fee2454e42a23756c",
    "index": 7,
    "guid": "6d5eeb9c-1c9b-4c94-9ade-7b85927e6d89",
    "isActive": true,
    "balance": "$3,923.75",
    "picture": "http://placehold.it/32x32",
    "age": 33,
    "eyeColor": "blue",
    "name": {
      "first": "Constance",
      "last": "Travis"
    },
    "company": "ZIDOX",
    "email": "constance.travis@zidox.biz",
    "phone": "+1 (918) 499-2733",
    "address": "243 Malbone Street, Wheaton, Iowa, 8798",
    "about": "Aliquip proident est et sunt magna. Duis ipsum velit laboris reprehenderit excepteur veniam. Sunt occaecat excepteur excepteur labore. Culpa sit aute ut id laborum commodo sint do qui enim magna ex anim. Voluptate ullamco veniam minim voluptate aute duis elit sunt voluptate in nisi dolore sit. Elit labore laboris amet incididunt in cupidatat non et est qui consectetur.",
    "registered": "Wednesday, January 20, 2016 9:41 PM",
    "latitude": "73.648824",
    "longitude": "-150.40766",
    "tags": [
      "duis",
      "est",
      "labore",
      "sint",
      "aute"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Terri Mccarthy"
      },
      {
        "id": 1,
        "name": "Amalia Blankenship"
      },
      {
        "id": 2,
        "name": "Salas Oneill"
      }
    ],
    "greeting": "Hello, Constance!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fbdb1a902f9ed2736",
    "index": 8,
    "guid": "fa4ab5e7-b357-4da7-9a1e-33103721f4ad",
    "isActive": true,
    "balance": "$3,439.45",
    "picture": "http://placehold.it/32x32",
    "age": 34,
    "eyeColor": "blue",
    "name": {
      "first": "Thelma",
      "last": "Woods"
    },
    "company": "ACCUSAGE",
    "email": "thelma.woods@accusage.co.uk",
    "phone": "+1 (983) 512-2649",
    "address": "116 Gunnison Court, Idledale, West Virginia, 1593",
    "about": "Adipisicing quis qui Lorem est dolore elit qui Lorem sit aliqua. Sint duis veniam ipsum ex pariatur culpa nisi ut proident deserunt Lorem. Nisi enim non velit excepteur commodo non consectetur voluptate dolor quis mollit.",
    "registered": "Wednesday, July 23, 2014 7:59 AM",
    "latitude": "46.749703",
    "longitude": "-30.646582",
    "tags": [
      "sit",
      "veniam",
      "ipsum",
      "do",
      "consectetur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Lynnette Tanner"
      },
      {
        "id": 1,
        "name": "Lynn Thompson"
      },
      {
        "id": 2,
        "name": "Kristen Manning"
      }
    ],
    "greeting": "Hello, Thelma!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  }
]'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testBigJsonTA

	self shouldPass! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testBigJsonTMO

	| tJSParser  |
	tJSParser := ParsecJsonGrammar new with: ParsecMixedOrderAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '[
  {
    "_id": "5829933f45655c137f72afb4",
    "index": 0,
    "guid": "1de3c7fa-9f5f-4ce2-bd00-1492485feea5",
    "isActive": true,
    "balance": "$3,545.59",
    "picture": "http://placehold.it/32x32",
    "age": 26,
    "eyeColor": "green",
    "name": {
      "first": "Reeves",
      "last": "Stanton"
    },
    "company": "CHILLIUM",
    "email": "reeves.stanton@chillium.io",
    "phone": "+1 (993) 405-2896",
    "address": "114 Lombardy Street, Tivoli, New York, 5382",
    "about": "Nulla eiusmod sunt cupidatat officia laborum exercitation duis aliquip Lorem laboris nostrud veniam. Sunt in magna consequat reprehenderit exercitation veniam ut consequat exercitation. Occaecat nisi aliquip ad exercitation dolore. Id laboris pariatur aliquip irure ut duis et sunt dolore dolore. Exercitation non aliqua excepteur sint magna laborum. Tempor velit quis ad in Lorem Lorem amet mollit voluptate duis reprehenderit sunt dolor. Dolor qui enim qui mollit labore sunt fugiat.",
    "registered": "Saturday, November 28, 2015 10:53 PM",
    "latitude": "-73.020167",
    "longitude": "134.786528",
    "tags": [
      "adipisicing",
      "irure",
      "velit",
      "in",
      "pariatur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Mccarthy Mcguire"
      },
      {
        "id": 1,
        "name": "Sadie Swanson"
      },
      {
        "id": 2,
        "name": "Hodge Moody"
      }
    ],
    "greeting": "Hello, Reeves!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f561d35c3308fbcd8",
    "index": 1,
    "guid": "f2601a6c-4999-44fb-aeb2-af7849905ca4",
    "isActive": false,
    "balance": "$2,760.73",
    "picture": "http://placehold.it/32x32",
    "age": 30,
    "eyeColor": "brown",
    "name": {
      "first": "Patti",
      "last": "Aguirre"
    },
    "company": "HYDROCOM",
    "email": "patti.aguirre@hydrocom.ca",
    "phone": "+1 (999) 589-2632",
    "address": "837 Adler Place, Brazos, New Mexico, 6246",
    "about": "Magna dolore in nulla esse sint occaecat excepteur deserunt culpa excepteur. Anim quis minim fugiat fugiat anim laborum. Pariatur culpa non id aliqua eu anim sint ad adipisicing tempor non adipisicing laboris. Dolore adipisicing pariatur sit occaecat ut. Aliqua dolor minim excepteur laborum cupidatat nisi est cupidatat ut exercitation. Incididunt est commodo esse ea sint reprehenderit est labore laboris ex minim proident Lorem do.",
    "registered": "Wednesday, October 19, 2016 5:22 PM",
    "latitude": "38.995324",
    "longitude": "-95.68088",
    "tags": [
      "sint",
      "amet",
      "sunt",
      "consectetur",
      "Lorem"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Briggs Chapman"
      },
      {
        "id": 1,
        "name": "Kelley Neal"
      },
      {
        "id": 2,
        "name": "Chandra William"
      }
    ],
    "greeting": "Hello, Patti!! You have 6 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fc55451f6efea29e2",
    "index": 2,
    "guid": "9eed2015-63f5-469b-bc4d-3f625bb742d0",
    "isActive": false,
    "balance": "$2,077.11",
    "picture": "http://placehold.it/32x32",
    "age": 31,
    "eyeColor": "blue",
    "name": {
      "first": "Clemons",
      "last": "Jarvis"
    },
    "company": "COMTRAK",
    "email": "clemons.jarvis@comtrak.org",
    "phone": "+1 (910) 404-2741",
    "address": "258 Dakota Place, Summerset, South Carolina, 2662",
    "about": "Cillum magna tempor voluptate consectetur ullamco enim mollit ullamco aliquip incididunt nisi cupidatat. Nostrud aute est nisi incididunt cupidatat ut dolor ad non nostrud. Dolore labore fugiat consequat consectetur incididunt cupidatat minim laboris. Ullamco est est incididunt nulla nulla minim enim occaecat fugiat ut tempor. Deserunt cupidatat nisi adipisicing est velit id amet qui. Commodo aliqua tempor occaecat ea consectetur consectetur.",
    "registered": "Monday, December 1, 2014 7:42 AM",
    "latitude": "22.819718",
    "longitude": "-159.31187",
    "tags": [
      "deserunt",
      "incididunt",
      "ipsum",
      "enim",
      "velit"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Little Rodgers"
      },
      {
        "id": 1,
        "name": "Sofia Cooley"
      },
      {
        "id": 2,
        "name": "Monroe Jennings"
      }
    ],
    "greeting": "Hello, Clemons!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f376548679c1f7fe3",
    "index": 3,
    "guid": "6e22aeee-d6ee-433f-977c-7bd1acd10273",
    "isActive": false,
    "balance": "$1,811.95",
    "picture": "http://placehold.it/32x32",
    "age": 32,
    "eyeColor": "green",
    "name": {
      "first": "Mathis",
      "last": "Leach"
    },
    "company": "COMTREK",
    "email": "mathis.leach@comtrek.me",
    "phone": "+1 (941) 469-2252",
    "address": "885 Pooles Lane, Morriston, Indiana, 1144",
    "about": "Anim voluptate id nulla officia irure. Anim sunt qui elit ipsum consequat nisi. Minim laborum eiusmod officia nostrud. Duis irure laborum sit deserunt nostrud occaecat pariatur ut consectetur incididunt quis tempor consequat aliquip. Eu excepteur qui fugiat voluptate sint pariatur aliqua commodo ea fugiat aliquip ipsum nulla. Aliquip enim anim commodo pariatur dolor eiusmod fugiat culpa aliquip labore esse officia labore.",
    "registered": "Tuesday, July 15, 2014 12:27 AM",
    "latitude": "-86.086873",
    "longitude": "-145.063901",
    "tags": [
      "aliqua",
      "nisi",
      "cupidatat",
      "voluptate",
      "enim"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Nixon Underwood"
      },
      {
        "id": 1,
        "name": "Sandoval Oneil"
      },
      {
        "id": 2,
        "name": "Jodie Bass"
      }
    ],
    "greeting": "Hello, Mathis!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f91080e43f2815a8f",
    "index": 4,
    "guid": "a110cbcd-da9c-44c5-9740-12795daa38a1",
    "isActive": true,
    "balance": "$3,948.74",
    "picture": "http://placehold.it/32x32",
    "age": 38,
    "eyeColor": "brown",
    "name": {
      "first": "Hardy",
      "last": "Pace"
    },
    "company": "EMOLTRA",
    "email": "hardy.pace@emoltra.biz",
    "phone": "+1 (823) 523-3553",
    "address": "150 Varick Street, Crawfordsville, Rhode Island, 1521",
    "about": "Amet sunt occaecat officia laboris exercitation. Id in amet nostrud eiusmod esse elit. Ullamco laborum occaecat ullamco ipsum qui dolore est pariatur mollit magna nostrud dolore cillum. Excepteur dolore ad consectetur magna pariatur eiusmod. Veniam exercitation proident deserunt magna. Ad consequat deserunt reprehenderit commodo nulla adipisicing id pariatur consequat exercitation do ullamco proident dolore.",
    "registered": "Saturday, February 20, 2016 3:42 AM",
    "latitude": "75.28317",
    "longitude": "176.558437",
    "tags": [
      "ex",
      "quis",
      "ex",
      "qui",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Macdonald Cleveland"
      },
      {
        "id": 1,
        "name": "Caldwell Brewer"
      },
      {
        "id": 2,
        "name": "Justine Beasley"
      }
    ],
    "greeting": "Hello, Hardy!! You have 9 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fe9ef1f88060d2e73",
    "index": 5,
    "guid": "5e034438-ae3a-4981-bc30-c07ccb4ba2c2",
    "isActive": false,
    "balance": "$3,161.10",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "blue",
    "name": {
      "first": "Martha",
      "last": "Spence"
    },
    "company": "COLLAIRE",
    "email": "martha.spence@collaire.net",
    "phone": "+1 (894) 513-2129",
    "address": "788 Crooke Avenue, Gardiner, Connecticut, 4632",
    "about": "Cupidatat fugiat id sint deserunt in et et aliquip qui velit do reprehenderit occaecat et. Nulla nostrud enim id anim irure elit enim fugiat nostrud voluptate est consequat. Voluptate laboris labore occaecat aliquip dolor. Voluptate ad fugiat laboris quis aute deserunt in aliqua.",
    "registered": "Wednesday, August 6, 2014 11:53 AM",
    "latitude": "21.811418",
    "longitude": "71.300384",
    "tags": [
      "incididunt",
      "pariatur",
      "cupidatat",
      "ea",
      "veniam"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Brown Williams"
      },
      {
        "id": 1,
        "name": "Deanna Ferguson"
      },
      {
        "id": 2,
        "name": "Violet Shaw"
      }
    ],
    "greeting": "Hello, Martha!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fc77a2bae9dc5145a",
    "index": 6,
    "guid": "e33f729a-d690-49ba-bc89-ca56f934e0ea",
    "isActive": false,
    "balance": "$3,451.86",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "brown",
    "name": {
      "first": "Stacy",
      "last": "Houston"
    },
    "company": "INSURON",
    "email": "stacy.houston@insuron.com",
    "phone": "+1 (854) 435-3498",
    "address": "959 Concord Street, Aurora, New Hampshire, 7772",
    "about": "Velit exercitation excepteur eiusmod exercitation aliqua duis cupidatat eiusmod aliquip pariatur laborum amet. Laborum adipisicing officia culpa enim pariatur fugiat id officia magna. Sit occaecat Lorem aliquip sint id laborum magna. Consectetur labore laborum reprehenderit magna nostrud adipisicing. Lorem deserunt Lorem sunt dolor. Dolore amet ut commodo nulla Lorem incididunt Lorem sunt labore laborum nisi consectetur.",
    "registered": "Tuesday, November 24, 2015 11:38 AM",
    "latitude": "77.866637",
    "longitude": "-57.468626",
    "tags": [
      "non",
      "esse",
      "velit",
      "deserunt",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Kimberley Mayer"
      },
      {
        "id": 1,
        "name": "Hampton Joyner"
      },
      {
        "id": 2,
        "name": "Bowen Freeman"
      }
    ],
    "greeting": "Hello, Stacy!! You have 9 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fee2454e42a23756c",
    "index": 7,
    "guid": "6d5eeb9c-1c9b-4c94-9ade-7b85927e6d89",
    "isActive": true,
    "balance": "$3,923.75",
    "picture": "http://placehold.it/32x32",
    "age": 33,
    "eyeColor": "blue",
    "name": {
      "first": "Constance",
      "last": "Travis"
    },
    "company": "ZIDOX",
    "email": "constance.travis@zidox.biz",
    "phone": "+1 (918) 499-2733",
    "address": "243 Malbone Street, Wheaton, Iowa, 8798",
    "about": "Aliquip proident est et sunt magna. Duis ipsum velit laboris reprehenderit excepteur veniam. Sunt occaecat excepteur excepteur labore. Culpa sit aute ut id laborum commodo sint do qui enim magna ex anim. Voluptate ullamco veniam minim voluptate aute duis elit sunt voluptate in nisi dolore sit. Elit labore laboris amet incididunt in cupidatat non et est qui consectetur.",
    "registered": "Wednesday, January 20, 2016 9:41 PM",
    "latitude": "73.648824",
    "longitude": "-150.40766",
    "tags": [
      "duis",
      "est",
      "labore",
      "sint",
      "aute"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Terri Mccarthy"
      },
      {
        "id": 1,
        "name": "Amalia Blankenship"
      },
      {
        "id": 2,
        "name": "Salas Oneill"
      }
    ],
    "greeting": "Hello, Constance!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fbdb1a902f9ed2736",
    "index": 8,
    "guid": "fa4ab5e7-b357-4da7-9a1e-33103721f4ad",
    "isActive": true,
    "balance": "$3,439.45",
    "picture": "http://placehold.it/32x32",
    "age": 34,
    "eyeColor": "blue",
    "name": {
      "first": "Thelma",
      "last": "Woods"
    },
    "company": "ACCUSAGE",
    "email": "thelma.woods@accusage.co.uk",
    "phone": "+1 (983) 512-2649",
    "address": "116 Gunnison Court, Idledale, West Virginia, 1593",
    "about": "Adipisicing quis qui Lorem est dolore elit qui Lorem sit aliqua. Sint duis veniam ipsum ex pariatur culpa nisi ut proident deserunt Lorem. Nisi enim non velit excepteur commodo non consectetur voluptate dolor quis mollit.",
    "registered": "Wednesday, July 23, 2014 7:59 AM",
    "latitude": "46.749703",
    "longitude": "-30.646582",
    "tags": [
      "sit",
      "veniam",
      "ipsum",
      "do",
      "consectetur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Lynnette Tanner"
      },
      {
        "id": 1,
        "name": "Lynn Thompson"
      },
      {
        "id": 2,
        "name": "Kristen Manning"
      }
    ],
    "greeting": "Hello, Thelma!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  }
]'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testBigJsonTO

	| tJSParser  |
	tJSParser := ParsecJsonGrammar new with: ParsecOrderedAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '[
  {
    "_id": "5829933f45655c137f72afb4",
    "index": 0,
    "guid": "1de3c7fa-9f5f-4ce2-bd00-1492485feea5",
    "isActive": true,
    "balance": "$3,545.59",
    "picture": "http://placehold.it/32x32",
    "age": 26,
    "eyeColor": "green",
    "name": {
      "first": "Reeves",
      "last": "Stanton"
    },
    "company": "CHILLIUM",
    "email": "reeves.stanton@chillium.io",
    "phone": "+1 (993) 405-2896",
    "address": "114 Lombardy Street, Tivoli, New York, 5382",
    "about": "Nulla eiusmod sunt cupidatat officia laborum exercitation duis aliquip Lorem laboris nostrud veniam. Sunt in magna consequat reprehenderit exercitation veniam ut consequat exercitation. Occaecat nisi aliquip ad exercitation dolore. Id laboris pariatur aliquip irure ut duis et sunt dolore dolore. Exercitation non aliqua excepteur sint magna laborum. Tempor velit quis ad in Lorem Lorem amet mollit voluptate duis reprehenderit sunt dolor. Dolor qui enim qui mollit labore sunt fugiat.",
    "registered": "Saturday, November 28, 2015 10:53 PM",
    "latitude": "-73.020167",
    "longitude": "134.786528",
    "tags": [
      "adipisicing",
      "irure",
      "velit",
      "in",
      "pariatur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Mccarthy Mcguire"
      },
      {
        "id": 1,
        "name": "Sadie Swanson"
      },
      {
        "id": 2,
        "name": "Hodge Moody"
      }
    ],
    "greeting": "Hello, Reeves!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f561d35c3308fbcd8",
    "index": 1,
    "guid": "f2601a6c-4999-44fb-aeb2-af7849905ca4",
    "isActive": false,
    "balance": "$2,760.73",
    "picture": "http://placehold.it/32x32",
    "age": 30,
    "eyeColor": "brown",
    "name": {
      "first": "Patti",
      "last": "Aguirre"
    },
    "company": "HYDROCOM",
    "email": "patti.aguirre@hydrocom.ca",
    "phone": "+1 (999) 589-2632",
    "address": "837 Adler Place, Brazos, New Mexico, 6246",
    "about": "Magna dolore in nulla esse sint occaecat excepteur deserunt culpa excepteur. Anim quis minim fugiat fugiat anim laborum. Pariatur culpa non id aliqua eu anim sint ad adipisicing tempor non adipisicing laboris. Dolore adipisicing pariatur sit occaecat ut. Aliqua dolor minim excepteur laborum cupidatat nisi est cupidatat ut exercitation. Incididunt est commodo esse ea sint reprehenderit est labore laboris ex minim proident Lorem do.",
    "registered": "Wednesday, October 19, 2016 5:22 PM",
    "latitude": "38.995324",
    "longitude": "-95.68088",
    "tags": [
      "sint",
      "amet",
      "sunt",
      "consectetur",
      "Lorem"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Briggs Chapman"
      },
      {
        "id": 1,
        "name": "Kelley Neal"
      },
      {
        "id": 2,
        "name": "Chandra William"
      }
    ],
    "greeting": "Hello, Patti!! You have 6 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fc55451f6efea29e2",
    "index": 2,
    "guid": "9eed2015-63f5-469b-bc4d-3f625bb742d0",
    "isActive": false,
    "balance": "$2,077.11",
    "picture": "http://placehold.it/32x32",
    "age": 31,
    "eyeColor": "blue",
    "name": {
      "first": "Clemons",
      "last": "Jarvis"
    },
    "company": "COMTRAK",
    "email": "clemons.jarvis@comtrak.org",
    "phone": "+1 (910) 404-2741",
    "address": "258 Dakota Place, Summerset, South Carolina, 2662",
    "about": "Cillum magna tempor voluptate consectetur ullamco enim mollit ullamco aliquip incididunt nisi cupidatat. Nostrud aute est nisi incididunt cupidatat ut dolor ad non nostrud. Dolore labore fugiat consequat consectetur incididunt cupidatat minim laboris. Ullamco est est incididunt nulla nulla minim enim occaecat fugiat ut tempor. Deserunt cupidatat nisi adipisicing est velit id amet qui. Commodo aliqua tempor occaecat ea consectetur consectetur.",
    "registered": "Monday, December 1, 2014 7:42 AM",
    "latitude": "22.819718",
    "longitude": "-159.31187",
    "tags": [
      "deserunt",
      "incididunt",
      "ipsum",
      "enim",
      "velit"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Little Rodgers"
      },
      {
        "id": 1,
        "name": "Sofia Cooley"
      },
      {
        "id": 2,
        "name": "Monroe Jennings"
      }
    ],
    "greeting": "Hello, Clemons!! You have 5 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f376548679c1f7fe3",
    "index": 3,
    "guid": "6e22aeee-d6ee-433f-977c-7bd1acd10273",
    "isActive": false,
    "balance": "$1,811.95",
    "picture": "http://placehold.it/32x32",
    "age": 32,
    "eyeColor": "green",
    "name": {
      "first": "Mathis",
      "last": "Leach"
    },
    "company": "COMTREK",
    "email": "mathis.leach@comtrek.me",
    "phone": "+1 (941) 469-2252",
    "address": "885 Pooles Lane, Morriston, Indiana, 1144",
    "about": "Anim voluptate id nulla officia irure. Anim sunt qui elit ipsum consequat nisi. Minim laborum eiusmod officia nostrud. Duis irure laborum sit deserunt nostrud occaecat pariatur ut consectetur incididunt quis tempor consequat aliquip. Eu excepteur qui fugiat voluptate sint pariatur aliqua commodo ea fugiat aliquip ipsum nulla. Aliquip enim anim commodo pariatur dolor eiusmod fugiat culpa aliquip labore esse officia labore.",
    "registered": "Tuesday, July 15, 2014 12:27 AM",
    "latitude": "-86.086873",
    "longitude": "-145.063901",
    "tags": [
      "aliqua",
      "nisi",
      "cupidatat",
      "voluptate",
      "enim"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Nixon Underwood"
      },
      {
        "id": 1,
        "name": "Sandoval Oneil"
      },
      {
        "id": 2,
        "name": "Jodie Bass"
      }
    ],
    "greeting": "Hello, Mathis!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933f91080e43f2815a8f",
    "index": 4,
    "guid": "a110cbcd-da9c-44c5-9740-12795daa38a1",
    "isActive": true,
    "balance": "$3,948.74",
    "picture": "http://placehold.it/32x32",
    "age": 38,
    "eyeColor": "brown",
    "name": {
      "first": "Hardy",
      "last": "Pace"
    },
    "company": "EMOLTRA",
    "email": "hardy.pace@emoltra.biz",
    "phone": "+1 (823) 523-3553",
    "address": "150 Varick Street, Crawfordsville, Rhode Island, 1521",
    "about": "Amet sunt occaecat officia laboris exercitation. Id in amet nostrud eiusmod esse elit. Ullamco laborum occaecat ullamco ipsum qui dolore est pariatur mollit magna nostrud dolore cillum. Excepteur dolore ad consectetur magna pariatur eiusmod. Veniam exercitation proident deserunt magna. Ad consequat deserunt reprehenderit commodo nulla adipisicing id pariatur consequat exercitation do ullamco proident dolore.",
    "registered": "Saturday, February 20, 2016 3:42 AM",
    "latitude": "75.28317",
    "longitude": "176.558437",
    "tags": [
      "ex",
      "quis",
      "ex",
      "qui",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Macdonald Cleveland"
      },
      {
        "id": 1,
        "name": "Caldwell Brewer"
      },
      {
        "id": 2,
        "name": "Justine Beasley"
      }
    ],
    "greeting": "Hello, Hardy!! You have 9 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fe9ef1f88060d2e73",
    "index": 5,
    "guid": "5e034438-ae3a-4981-bc30-c07ccb4ba2c2",
    "isActive": false,
    "balance": "$3,161.10",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "blue",
    "name": {
      "first": "Martha",
      "last": "Spence"
    },
    "company": "COLLAIRE",
    "email": "martha.spence@collaire.net",
    "phone": "+1 (894) 513-2129",
    "address": "788 Crooke Avenue, Gardiner, Connecticut, 4632",
    "about": "Cupidatat fugiat id sint deserunt in et et aliquip qui velit do reprehenderit occaecat et. Nulla nostrud enim id anim irure elit enim fugiat nostrud voluptate est consequat. Voluptate laboris labore occaecat aliquip dolor. Voluptate ad fugiat laboris quis aute deserunt in aliqua.",
    "registered": "Wednesday, August 6, 2014 11:53 AM",
    "latitude": "21.811418",
    "longitude": "71.300384",
    "tags": [
      "incididunt",
      "pariatur",
      "cupidatat",
      "ea",
      "veniam"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Brown Williams"
      },
      {
        "id": 1,
        "name": "Deanna Ferguson"
      },
      {
        "id": 2,
        "name": "Violet Shaw"
      }
    ],
    "greeting": "Hello, Martha!! You have 10 unread messages.",
    "favoriteFruit": "apple"
  },
  {
    "_id": "5829933fc77a2bae9dc5145a",
    "index": 6,
    "guid": "e33f729a-d690-49ba-bc89-ca56f934e0ea",
    "isActive": false,
    "balance": "$3,451.86",
    "picture": "http://placehold.it/32x32",
    "age": 21,
    "eyeColor": "brown",
    "name": {
      "first": "Stacy",
      "last": "Houston"
    },
    "company": "INSURON",
    "email": "stacy.houston@insuron.com",
    "phone": "+1 (854) 435-3498",
    "address": "959 Concord Street, Aurora, New Hampshire, 7772",
    "about": "Velit exercitation excepteur eiusmod exercitation aliqua duis cupidatat eiusmod aliquip pariatur laborum amet. Laborum adipisicing officia culpa enim pariatur fugiat id officia magna. Sit occaecat Lorem aliquip sint id laborum magna. Consectetur labore laborum reprehenderit magna nostrud adipisicing. Lorem deserunt Lorem sunt dolor. Dolore amet ut commodo nulla Lorem incididunt Lorem sunt labore laborum nisi consectetur.",
    "registered": "Tuesday, November 24, 2015 11:38 AM",
    "latitude": "77.866637",
    "longitude": "-57.468626",
    "tags": [
      "non",
      "esse",
      "velit",
      "deserunt",
      "labore"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Kimberley Mayer"
      },
      {
        "id": 1,
        "name": "Hampton Joyner"
      },
      {
        "id": 2,
        "name": "Bowen Freeman"
      }
    ],
    "greeting": "Hello, Stacy!! You have 9 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fee2454e42a23756c",
    "index": 7,
    "guid": "6d5eeb9c-1c9b-4c94-9ade-7b85927e6d89",
    "isActive": true,
    "balance": "$3,923.75",
    "picture": "http://placehold.it/32x32",
    "age": 33,
    "eyeColor": "blue",
    "name": {
      "first": "Constance",
      "last": "Travis"
    },
    "company": "ZIDOX",
    "email": "constance.travis@zidox.biz",
    "phone": "+1 (918) 499-2733",
    "address": "243 Malbone Street, Wheaton, Iowa, 8798",
    "about": "Aliquip proident est et sunt magna. Duis ipsum velit laboris reprehenderit excepteur veniam. Sunt occaecat excepteur excepteur labore. Culpa sit aute ut id laborum commodo sint do qui enim magna ex anim. Voluptate ullamco veniam minim voluptate aute duis elit sunt voluptate in nisi dolore sit. Elit labore laboris amet incididunt in cupidatat non et est qui consectetur.",
    "registered": "Wednesday, January 20, 2016 9:41 PM",
    "latitude": "73.648824",
    "longitude": "-150.40766",
    "tags": [
      "duis",
      "est",
      "labore",
      "sint",
      "aute"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Terri Mccarthy"
      },
      {
        "id": 1,
        "name": "Amalia Blankenship"
      },
      {
        "id": 2,
        "name": "Salas Oneill"
      }
    ],
    "greeting": "Hello, Constance!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  },
  {
    "_id": "5829933fbdb1a902f9ed2736",
    "index": 8,
    "guid": "fa4ab5e7-b357-4da7-9a1e-33103721f4ad",
    "isActive": true,
    "balance": "$3,439.45",
    "picture": "http://placehold.it/32x32",
    "age": 34,
    "eyeColor": "blue",
    "name": {
      "first": "Thelma",
      "last": "Woods"
    },
    "company": "ACCUSAGE",
    "email": "thelma.woods@accusage.co.uk",
    "phone": "+1 (983) 512-2649",
    "address": "116 Gunnison Court, Idledale, West Virginia, 1593",
    "about": "Adipisicing quis qui Lorem est dolore elit qui Lorem sit aliqua. Sint duis veniam ipsum ex pariatur culpa nisi ut proident deserunt Lorem. Nisi enim non velit excepteur commodo non consectetur voluptate dolor quis mollit.",
    "registered": "Wednesday, July 23, 2014 7:59 AM",
    "latitude": "46.749703",
    "longitude": "-30.646582",
    "tags": [
      "sit",
      "veniam",
      "ipsum",
      "do",
      "consectetur"
    ],
    "range": [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ],
    "friends": [
      {
        "id": 0,
        "name": "Lynnette Tanner"
      },
      {
        "id": 1,
        "name": "Lynn Thompson"
      },
      {
        "id": 2,
        "name": "Kristen Manning"
      }
    ],
    "greeting": "Hello, Thelma!! You have 5 unread messages.",
    "favoriteFruit": "banana"
  }
]'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testExpressionGrammar

	|result|

	result := ParsecExprGrammar new 
		parse: '2*3+4' using: ParsecMixedOrderAlgo new log.
		
	self assert: result succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testFMOcontextSensitive

	charts := ParsecMixedOrderAlgo new log
		parse: 'aaabbbccc'
		using: ParsecGrammar contextSensitive.
		
	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testFMOcontextSensitiveWithNot

	charts := ParsecMixedOrderAlgo new log
		parse: 'aaabbbccc'
		using: ParsecGrammar contextSensitiveWithNot.
		
	self assert: charts succeeded.
	
	charts := ParsecMixedOrderAlgo new log
		parse: 'aaabbbcccddd' "staro with not and pluso"
		using: ParsecGrammar contextSensitiveWithNot.
		
	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testFMOenglisOpsMixed

	charts := ParsecMixedOrderAlgo new log
		parse: 'book that flight   '
		using: ParsecGrammar englishWithTempsEmptyInlineTerminalsAndOpsMixed.
		
	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testFMOenglisOpsMixedWithError


	charts := ParsecMixedOrderAlgo new log
		parse: 'book that flight   8'
		using: ParsecGrammar englishWithTempsEmptyInlineTerminalsAndOpsMixed.
		
	self assert: charts succeeded not.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testFMOnumberFloat

	charts := ParsecMixedOrderAlgo new log
		parse: '23.23'
		using: ParsecGrammar numberFloat.
		
	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 1/11/2017 21:26:53'!
testFilteredMOcontextSensitiveWithNot

	charts := ParsecMixedOrderFilteredAlgo new log
		parse: 'aaabbbccc'
		using: ParsecGrammar contextSensitiveWithNot.
		
	self assert: charts succeeded.
	
	charts := ParsecMixedOrderFilteredAlgo new log
		parse: 'aaabbbcccddd' "staro with not and pluso"
		using: ParsecGrammar contextSensitiveWithNot.
		
	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testJson

	| tJSParser  |
	tJSParser := ParsecJsonLoggingGrammar new. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testJsonTA

	| tJSParser  |
	tJSParser := ParsecJsonGrammar new with: ParsecAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testJsonTM

	| tJSParser  |
	tJSParser := ParsecJsonLoggingGrammar new with: ParsecMixedOrderAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testJsonTMF

	| tJSParser  |
	tJSParser := ParsecJsonLoggingGrammar new with: ParsecMixedOrderFilteredAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: 'Nick Papoylias 3/13/2017 20:20:40'!
testJsonTO

	| tJSParser  |
	tJSParser := ParsecJsonGrammar new with: ParsecOrderedAlgo. "remember that the grammar is created its time -- for bench etc"

	charts := tJSParser parse: '{ "firstName" : "John",
"lastName" : "Smith",
"age" : 25,
"address" :
{ "streetAddress" : "21 2nd Street",
"city" : "New York",
"state" : "NY",
"postalCode" : "10021" },
"phoneNumber":
[
{ "type" : "home",
"number" : "212 555-1234" },
{ "type" : "fax",
"number" : "646 555-4567" } ] }'.

	self assert: charts succeeded.! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testNonPeg

	charts := ParsecMixedOrderAlgo new log
		parse: 'xxxxxxx'
		using: ParsecGrammar nonPeg.
		
	self assert: charts succeeded! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testNonPegOrd

	charts := ParsecMixedOrderAlgo new log
		parse: 'xxxxxxx'
		using: ParsecGrammar nonPegOrd.
		
	self assert: charts succeeded! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testNonPegOrdJustO

	charts := ParsecOrderedAlgo new log
		parse: 'xxxxxxx'
		using: ParsecGrammar nonPegOrd.
		
	self assert: charts succeeded! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testNumberFloatRegEx

	charts := ParsecAlgo new log
		parse: '23.23'
		using: ParsecGrammar numberFloatRegEx.
		
	self assert: charts succeeded! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testOcontextSensitive
	
	charts := ParsecOrderedAlgo new log
		parse: 'aaabbbccc'
		using: ParsecGrammar contextSensitive.

	self assert: charts succeeded! !

!ParsecAlgosTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testOnumberFloatOrdered

	charts := ParsecOrderedAlgo new log
		parse: '23.23'
		using: ParsecGrammar numberFloatOrdered.
		
	self assert: charts succeeded! !

TestCase subclass: #ParsecCollectionTest
	instanceVariableNames: 'charts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecCollectionTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
classToTest

	^ ParsecCollection! !

!ParsecCollectionTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testAnySatisfy
	|t|
	t := self classToTest new.
	ProtoObject allSubclasses do: [:c | t add: c. ].
	self assert: (t anySatisfy: [ :c | c == Object ]).
	self assert: (t anySatisfy: [ :c | c == self classToTest ]).
	self deny: (t anySatisfy: [ :c | c == ProtoObject ]).! !

!ParsecCollectionTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testBench
	|t o|
	t := self classToTest new.
	o := OrderedCollection new.

	Transcript show:
		([1 to: 200000 do: [:i | OrderedCollection new]] timeToRun /
		[1 to: 200000 do: [:i |  self classToTest new ]] timeToRun) asFloat; cr.

	Transcript show:
		([1 to: 200000 do: [:i | o add: i ]] timeToRun /
		[1 to: 200000 do: [:i | t add: i ]] timeToRun) asFloat; cr.

	Transcript show:
		([1 to: 200000 do: [:i | o at: i ]] timeToRun /
		[1 to: 200000 do: [:i | t at: i ]] timeToRun) asFloat; cr.

	Transcript show:
		([1 to: 200000 do: [:i | o at: i put: i ]] timeToRun /
		[1 to: 200000 do: [:i | t at: i put: i ]] timeToRun) asFloat; cr.

	Transcript show: 
		([1 to: 200000 do: [:i | o at: i ifAbsent: [  ] ]] timeToRun /
		[1 to: 200000 do: [:i | t at: i ifAbsent: [  ] ]] timeToRun) asFloat; cr.
	
	Transcript show: 
		([1 to: 2000 do: [:i | o removeIndex: 1]] timeToRun /
		[1 to: 2000 do: [:i | t removeIndex: 1 ]] timeToRun) asFloat; cr.! !

!ParsecCollectionTest methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testInsertionAndRemoval
	|t|
	
	"should break into multiple tests, this also tests at:, at:put: etc.."
	
	t :=  self classToTest new.
	1 to: 20000 do: [:i | "should also trigger grow"
		t add: i. 
		self assert: t last = i.
		self assert: t size = i.
	].
	t removeIndex: 20000.
	self assert: t last = 19999.
	self assert: t size = 19999.
	1 to: 19998 do: [:i | 
		t removeIndex: 1. 
		self assert: t first = (i + 1).
		self assert: t size = (19999 - i)
	].
	self assert: t last = 19999.
	self assert: t size = 1.! !

TestCase subclass: #ParsecTestFirstFollow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec'!

!ParsecTestFirstFollow methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testFirstFollowSetEquivalence

	|grammar|

	grammar := ParsecGrammar firstFollowExample.
	self assert: (grammar at: #E) firstsAsStrings = (grammar at: #F) firstsAsStrings.
	self assert: (grammar at: #E) firstsAsStrings = (grammar at: #T) firstsAsStrings.
	self assert: (grammar at: #E) followAsStrings = (grammar at: #E_) followAsStrings.
	self assert: (grammar at: #T) followAsStrings = (grammar at: #T_) followAsStrings.
! !

!ParsecTestFirstFollow methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testFirstFollowSetTerminals

	|grammar|

	grammar := ParsecGrammar firstFollowExample.
	self assert: (grammar at: #E) firstsAsStrings = { '\(' . '[a-zA-Z][a-zA-Z0-9]*' } asSet.
	self assert: (grammar at: #E_) firstsAsStrings = { '\+' . #e } asSet.
	self assert: (grammar at: #T_) firstsAsStrings = { '\*' . #e } asSet.
	self assert: (grammar at: #E) followAsStrings = {'\)' . #Eof} asSet.	
	self assert: (grammar at: #T) followAsStrings = {'\+'. '\)' . #Eof} asSet.	
	self assert: (grammar at: #F) followAsStrings = {'\*' . '\+' . '\)' . #Eof} asSet.	! !

TestCase subclass: #TestParsecLexer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBench

	| oldLex newLex result |
	oldLex := '[0-9]+' asRegex.
	newLex := '[0-9]+' asParsecRegex.
	result := [1 to: 500000 do: [:i | oldLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 500000 do: [:i | newLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat.
	self assert: result > 1.
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBenchDFA

	| oldLex newLex result |
	oldLex := '[0-9]+' asParsecRegex.
	newLex := '[0-9]+' asParsecRegex asDFA.
	result := [1 to: 500000 do: [:i | oldLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 500000 do: [:i | newLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat.
	"self assert: result > 1."
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBenchNFA

	| oldLex newLex result |
	oldLex := '[0-9]+' asParsecRegex.
	newLex := '[0-9]+' asParsecRegex asNFA.
	result := [1 to: 500000 do: [:i | oldLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 500000 do: [:i | newLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun asNanoSeconds asFloat.
	"self assert: result > 1."
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBenchString

	| oldLex newLex result |
	oldLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asRegex.
	newLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asParsecRegex.
	result := [1 to: 50000 do: [:i | oldLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 50000 do: [:i | newLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat.
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBenchStringDFA

	| oldLex newLex result |
	oldLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asParsecRegex.
	newLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asParsecRegex asDFA.
	result := [1 to: 50000 do: [:i | oldLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 50000 do: [:i | newLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat.
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/10/2017 20:14:21'!
testBenchStringNFA

	| oldLex newLex result |
	oldLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asParsecRegex.
	newLex := '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"' asParsecRegex asNFA.
	result := [1 to: 50000 do: [:i | oldLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat /
	[1 to: 50000 do: [:i | newLex matchesStreamPrefix: '"hallo world\n"' readStream]] timeToRun asNanoSeconds asFloat.
	Transcript show: result; cr.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testCompositeReg
	
	|result|

	result := (ParsecLexer new parse: 'asdf+aaa?|a*(abc|d)?').
	self assert: result class = ParsecLexerOption.
	self assert: result first class = ParsecLexerConcat.
	self assert: result first first class = ParsecLexerPlus.
	self assert: result first first operand operand = 'asdf'.
	self assert: result first second class = ParsecLexerOpt.
	self assert: result first second operand operand = 'aaa'.
	self assert: result second class = ParsecLexerConcat.
	self assert: result second first class = ParsecLexerStar.
	self assert: result second first operand operand = $a.
	self assert: result second second class = ParsecLexerOpt.
	self assert: result second second operand class = ParsecLexerOption.	
	self assert: result second second operand first class = ParsecLexerOperand.	
	self assert: result second second operand first operand = 'abc'.	
	self assert: result second second operand second class = ParsecLexerOperand.	
	self assert: result second second operand second operand = $d.	
	! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testCompositeRegWithExtentions
	
	|result|

	result := (ParsecLexer new parse: 'asd.\n[a-zA-Z]+aaa?|a{3}(abc|d){2,5}').
	self assert: result class = ParsecLexerOption.
	self assert: result first class = ParsecLexerConcat.
	self assert: result first first class = ParsecLexerOperand.
	self assert: result first first operand = 'asd'.
	self assert: result first second class = ParsecLexerAny.
	self assert: result first second operand = nil.
	self assert: result first third class = ParsecLexerEscape.
	self assert: result first fourth class = ParsecLexerPlus.
	"self assert: result first fourth operand class = TotLexerOption." "flattened"
	self assert: result second class = ParsecLexerConcat.
	self assert: result second first class = ParsecLexerRepetition.
	self assert: result second first operand operand = $a.
	self assert: result second second class = ParsecLexerRepetition.
	self assert: result second second operand class = ParsecLexerOption.	
	self assert: result second second operand first class = ParsecLexerOperand.	
	self assert: result second second operand first operand = 'abc'.	
	self assert: result second second operand second class = ParsecLexerOperand.	
	self assert: result second second operand second operand = $d.	
	! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testMatch
	
	|result|
	
	"It's faster.. Should further optimize (NFA/DFA): oldLex := '(1|2|3|4|5|6|7|8|9|0)+' asRegex.
newLex := (TotLexer new parse: '(1|2|3|4|5|6|7|8|9|0)+').
[1 to: 500000 do: [:i | newLex lexMatch: '324525 sfsdf' readStream]] timeToRun.
[1 to: 500000 do: [:i |oldLex matchesStreamPrefix: '324525 sfsdf' readStream]] timeToRun.
"

	result := (ParsecLexer new parse: 'asdf+aaa?|a+(abc|d)?').
	self assert: (result matchAtEnd: 'asdfasdf').
	self assert: (result matchAtEnd: 'asdf').
	self assert: (result matchAtEnd: 'asdfasdfaaa').
	self assert: (result match: 'asdfasdfa').
	self assert: (result matchAtEnd: 'asdfasdfb') not.
	self assert: (result matchAtEnd: 'asdg') not.
	self assert: (result matchAtEnd: 'asdfasdfa') not.
	self assert: (result match: 'aaaaa').
	self assert: (result matchAtEnd: 'aaaaag') not.
	self assert: (result matchAtEnd: 'aaaaabc') not. "ha !! -- should match as DFA/NFA"
	self assert: (result match: 'aaaaad'). 
	self assert: (result matchAtEnd: 'aaaaae') not.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testString

	|result|	
	
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: result asList = {#ParsecLexerConcat. {
						{#ParsecLexerOperand. $"}. 
						{#ParsecLexerStar. 
							{#ParsecLexerOption. {
								{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $/}. 
										{#ParsecLexerOperand. $"}. 
										{#ParsecLexerOperand. $b}. 
										{#ParsecLexerOperand. $f}. 
										{#ParsecLexerOperand. $n}. 
										{#ParsecLexerOperand. $r}. 
										{#ParsecLexerOperand. $t}}}}}}. 
						 		{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerOperand. $u}. 
									{#ParsecLexerRepetition. { #ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerRange. $0->$9}. 
										{#ParsecLexerRange. $a->$f}. 
										{#ParsecLexerRange. $A->$F}}}}}}}. 
								{#ParsecLexerNotGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $"}}}}}}}. 
						{#ParsecLexerOperand. $"}}}.
		self assert: (result match: '"hallo world\n"').
		self assert: (result match: '"hallo world\n') not.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testStringDFA

	|result|	
	
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: result asList = {#ParsecLexerConcat. {
						{#ParsecLexerOperand. $"}. 
						{#ParsecLexerStar. 
							{#ParsecLexerOption. {
								{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $/}. 
										{#ParsecLexerOperand. $"}. 
										{#ParsecLexerOperand. $b}. 
										{#ParsecLexerOperand. $f}. 
										{#ParsecLexerOperand. $n}. 
										{#ParsecLexerOperand. $r}. 
										{#ParsecLexerOperand. $t}}}}}}. 
						 		{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerOperand. $u}. 
									{#ParsecLexerRepetition. { #ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerRange. $0->$9}. 
										{#ParsecLexerRange. $a->$f}. 
										{#ParsecLexerRange. $A->$F}}}}}}}. 
								{#ParsecLexerNotGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $"}}}}}}}. 
						{#ParsecLexerOperand. $"}}}.
		self assert: (result match: '"hallo world\n"').
		self assert: (result match: '"hallo world\n') not.
		self assert: (result asDFA match: '"hallo world\n"').
		self assert: (result asNFA match: '"hallo "world\n"').
		self assert: (result asDFA match: '"hallo world\n') not.! !

!TestParsecLexer methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testStringNFA

	|result|	
	
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: result asList = {#ParsecLexerConcat. {
						{#ParsecLexerOperand. $"}. 
						{#ParsecLexerStar. 
							{#ParsecLexerOption. {
								{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $/}. 
										{#ParsecLexerOperand. $"}. 
										{#ParsecLexerOperand. $b}. 
										{#ParsecLexerOperand. $f}. 
										{#ParsecLexerOperand. $n}. 
										{#ParsecLexerOperand. $r}. 
										{#ParsecLexerOperand. $t}}}}}}. 
						 		{#ParsecLexerConcat. {
									{#ParsecLexerEscape. $\}. 
									{#ParsecLexerOperand. $u}. 
									{#ParsecLexerRepetition. { #ParsecLexerGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerRange. $0->$9}. 
										{#ParsecLexerRange. $a->$f}. 
										{#ParsecLexerRange. $A->$F}}}}}}}. 
								{#ParsecLexerNotGroup. 
									{#ParsecLexerOption. {
										{#ParsecLexerEscape. $\}. 
										{#ParsecLexerOperand. $"}}}}}}}. 
						{#ParsecLexerOperand. $"}}}.
		self assert: (result match: '"hallo world\n"').
		self assert: (result match: '"hallo world\n') not.
		self assert: (result asNFA match: '"hallo world\n"').
		self assert: (result asNFA match: '"hallo "world\n"').
		self assert: (result asNFA match: '"hallo world\n') not.! !

TestCase subclass: #TestToNFADFA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsec-Lexing'!

!TestToNFADFA methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testStringDFAParts

	|result|	
	"start with this an build your way through"
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt])*"').
	self assert: (result asDFA match: '"\"\\\/\b\f\n\r\t\n"').
	result := (ParsecLexer new parse: '"[^\\"]*"').
	self assert: (result asDFA match: '"meh"').
	result := (ParsecLexer new parse: '"(\\u[0-9a-fA-F]{4})"').
	self assert: (result asDFA match: '"\u0aBc"').
	result := (ParsecLexer new parse: '"(\\u[0-9a-fA-F]{4})+"').
	self assert: (result asDFA match: '"\u0aBc\u1dfE"').
	result := (ParsecLexer new parse: 'a*b*').
	self assert: (result asDFA match: 'sdfasdf'). "matched null string"
	result := (ParsecLexer new parse: 'a*b*').
	self assert: (result asDFA match: 'aaaaa').
	result := (ParsecLexer new parse: '"(a*)"').
	self assert: (result asDFA match: '"aaaaa"').
	result := (ParsecLexer new parse: '"(a*)(b*)').
	self assert: (result asDFA match: '"aaaaa').
	result := (ParsecLexer new parse: '"(a*)(b*)"').
	self assert: (result asDFA match: '"aaaaa"').
	result := (ParsecLexer new parse: '"(\\u[0-9a-fA-F]{4})*"').
	self assert: (result asDFA match: '"\u0aBc\u1dfE"').
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: (result asDFA match: '"hallo world\n"').
	
	
! !

!TestToNFADFA methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testStringNFAParts

	|result|	
	"start with this an build your way through"
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt])*"').
	self assert: (result asNFA match: '"\"\\\/\b\f\n\r\t\n"').
	result := (ParsecLexer new parse: '"[^\\"]*"').
	self assert: (result asNFA match: '"meh"').
	result := (ParsecLexer new parse: 'a*b*').
	self assert: (result asNFA match: 'sdfasdf'). "matched null string"
	result := (ParsecLexer new parse: '"(\\u[0-9a-fA-F]{4})*"').
	self assert: (result asNFA match: '"\u0aBc\u1dfE"').
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: (result asNFA match: '"hallo world\n"').

	
! !

!TestToNFADFA methodsFor: 'tests' stamp: ' 1/8/2017 13:28:27'!
testStringParts

	|result|	
	"start with this an build your way through"
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt])*"').
	self assert: (result match: '"\"\\\/\b\f\n\r\t\n"').
	result := (ParsecLexer new parse: '"[^\\"]*"').
	self assert: (result match: '"meh"').
	result := (ParsecLexer new parse: '"(\\u[0-9a-fA-F]{4})*"').
	self assert: (result match: '"\u0aBc\u1dfE"').
	result := (ParsecLexer new parse: '"(\\[\\/"bfnrt]|\\u[0-9a-fA-F]{4}|[^\\"])*"').
	self assert: (result match: '"hallo world\n"').

	
! !

!UndefinedObject methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
between: a and: b

	^ false! !

!Array methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:50'!
and
	
	|name|
	name := ('ta' , self hash asString) asSymbol.
	^  (name --|>  self) and. "returns rule"! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
asRule

	| rule op|

	rule := ParsecSubRule new name: ('t' , self hash asString) asSymbol.

	op := #,.
	
	self doWithIndex: [ :item :index | "catch unary" 
		(self catchUnaryFor: rule at: index given: op)
			ifFalse: [  
				(ParsecGrammar unaryOps includes: item) "ignore cause it was handled as unary"
					ifFalse: [  
						(item isSymbol and: [ParsecGrammar binaryOps includes: item])
							ifTrue: [ op := item ]
							ifFalse: [  rule perform: op withArguments: {item} ]
				]
			]
	].

	^ rule! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
asSelfEvaluatingForm
	
	|s|
	s := String new writeStream.
	self printRecursivelyAsSelfEvaluatingFormOn: s.
	^ s contents! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
catchUnaryFor: aRule at: anIndex given: anOp

	^ (anIndex < self size)
		ifTrue: [ |unaryOp | 
			(ParsecGrammar unaryOps includes: (unaryOp := (self at: anIndex + 1)))
				ifTrue: [ aRule perform: anOp withArguments: { (self at: anIndex) perform: unaryOp }. true ]
				ifFalse: [ false ]
		] ifFalse: [ false ]
! !

!Array methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:50'!
not
	
	|name|
	name := ('tn' , self hash asString) asSymbol.
	^  (name --|>  self) not. "returns rule"! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opt

	"A opt = #(A | e)"
	
	^ { self . #| . #e }! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
opto

	"A opt = #(A | e)"
	
	^ { self . #\ . #e }! !

!Array methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:50'!
plus

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #| . #e} . "returns rule"! !

!Array methodsFor: '*Parsec' stamp: 'NickPapoulias 1/17/2017 14:50'!
pluso

	"A+ = #(A , A*)"
	
	|name|
	name := ('tp' , self hash asString) asSymbol.
	^  name --|>  self , {name . #\ . #e} . "returns rule"! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
printRecursivelyAsSelfEvaluatingFormOn: aStream

	aStream nextPut: ${.
	self do: [:el | el printRecursivelyAsSelfEvaluatingFormOn: aStream] separatedBy: [ aStream nextPutAll: '. '].
	aStream nextPut: $}! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
star

	"A* = #(A+ | e)"
	
	^ { self plus . #| . #e }! !

!Array methodsFor: '*Parsec' stamp: ' 1/8/2017 13:28:27'!
staro

	"A* = #(A+ | e)"
	
	^ { self pluso . #\ . #e }! !
